@@ -751,6 +751,28 @@ BoardStatus BitBoard::validate() const
     return Valid;
 }
 
+bool BitBoard::canTakeEnPassant() const
+{
+    if(m_stm == Black)
+    {
+        quint64 movers = m_pawns & m_occupied_co[Black];
+        if(m_epSquare != NoEPSquare)
+        {
+            quint64 moves = bb_PawnAttacks[White][m_epSquare] & movers;
+            return (moves != 0);
+        }
+    }
+    else
+    {
+        quint64 movers = m_pawns & m_occupied_co[White];
+        if(m_epSquare != NoEPSquare)
+        {
+            quint64 moves = bb_PawnAttacks[Black][m_epSquare] & movers;
+            return (moves != 0);
+        }
+    }
+    return 0;
+}
 
 // Why QString throws asserts for access past end of string and
 // refuses to return a real c++ char type is beyond me...
@@ -1406,7 +1428,6 @@ Move BitBoard::parseMove(const QString& algebraic) const
         }
     }
 
-
     // Piece
     switch(c)
     {
@@ -1432,9 +1453,11 @@ Move BitBoard::parseMove(const QString& algebraic) const
         break;
     case 'P':
         c = *(s++); // Fall through
-    default:
         type = Pawn;
         break;
+    default:
+        type = Empty;
+        break;
     }
 
     // Check for disambiguation
@@ -1484,6 +1507,21 @@ Move BitBoard::parseMove(const QString& algebraic) const
         return move;
     }
 
+    if (type == Empty)
+    {
+        // either a pawn or a piece, is determined by source location
+        if (fromSquare < 0)
+            type = Pawn;
+        else
+        {
+            type = m_piece[fromSquare];
+            if (type == Empty)
+            {
+                return move; // There is no piece here
+            }
+        }
+    }
+
     if(type == Pawn)
     {
         // Promotion as in bxc8=Q or bxc8(Q)
@@ -1505,7 +1543,15 @@ Move BitBoard::parseMove(const QString& algebraic) const
         case 'N':
             promotePiece = Knight;
             break;
-        default: ;//return move;
+        default:
+            {
+                int lastRank = (m_stm == White ? 7 : 0);
+                if (lastRank == Rank(toSquare))
+                {
+                    return move;
+                }
+            }
+            break;
         }
         if(fromSquare < 0)
         {
