@@ -13,15 +13,20 @@
 #include "game.h"
 
 #include <QString>
+#include <QObject>
+#include <QUndoCommand>
+
 
 class Database;
 class Filter;
 class QUndoStack;
+class QUndoGroup;
 
 /** @ingroup Database
 The DatabaseInfo class is a simple class to keep various database-related
 information together. */
 
+
 class DatabaseInfo: public QThread
 {
     Q_OBJECT
@@ -29,9 +34,9 @@ public:
     void run();
     enum {NewGame = -1};
     /** Create information for clipboard database */
-    DatabaseInfo();
+    DatabaseInfo(QUndoGroup *undoGroup);
     /** Create information for file database */
-    DatabaseInfo(const QString& filename);
+    DatabaseInfo(QUndoGroup *undoGroup, const QString& filename);
     /** Close database and free memory */
     ~DatabaseInfo();
     /** Open database. */
@@ -68,7 +73,7 @@ public:
         return m_index;
     }
     /** Load game @p index */
-    bool loadGame(int index, bool reload = 0);
+    bool loadGame(int index);
     /** Starts new game, giving it @p index equal to the number of games in the database */
     void newGame();
     /** Saves game, replacing current one or adding new. If @ref currentIndex() is
@@ -82,20 +87,36 @@ public:
     {
         return m_filename;
     }
-    bool IsLoaded() const
-    {
-        return m_bLoaded;
-    }
-    bool IsUtf8() const
-    {
-        return m_utf8;
-    }
+
+    bool IsLoaded() const;
+    bool IsUtf8() const;
+
+    bool IsPGN() const;
+    bool IsPolyglotBook() const;
+
+    bool modified() const;
+    bool gameNeedsSaving() const;
+
+    void restoreState(const Game& game);
+
+    QUndoStack *undoStack() const;
+
 
 protected:
     void doLoadFile(QString filename);
 
 signals:
     void LoadFinished(DatabaseInfo*);
+    void signalRestoreState(const Game &game);
+    void signalGameModified();
+    void signalMoveChanged();
+    void searchProgress(int);
+    void searchFinished();
+
+
+public slots:
+    void dbCleanChanged(bool);
+    void setModified(bool modfied, const Game &g, QString action);
 
 private:
     QUndoStack* m_undoStack;
@@ -106,6 +127,28 @@ private:
     int m_index;
     bool m_bLoaded;
     bool m_utf8;
+    bool m_gameModified;
+};
+
+class GameUndoCommand : public QUndoCommand
+{
+public:
+    GameUndoCommand(QObject* parent, const Game& from, Game& to, QString action) :
+        QUndoCommand(action),
+        m_dbInfo((DatabaseInfo*)parent),
+        m_fromGame(from),
+        m_toGame(to),
+        m_bInConstructor(true)
+        {
+        }
+
+    QPointer<DatabaseInfo> m_dbInfo;
+    Game m_fromGame;
+    Game m_toGame;
+    bool m_bInConstructor;
+
+    virtual void undo() { m_dbInfo->restoreState(m_fromGame); }
+    virtual void redo() { if (m_bInConstructor) m_bInConstructor=false; else m_dbInfo->restoreState(m_toGame); }
 };
 
 #endif
