@@ -18,13 +18,11 @@ Game::Game() : QObject()
 {
     clearTags();
     clear();
-    setModified(false);
 }
 
 Game::Game(const Game& game) : QObject()
 {
     *this = game;
-    setModified(false);
 }
 
 Game& Game::operator=(const Game & game)
@@ -40,16 +38,11 @@ Game& Game::operator=(const Game & game)
         m_annotations = game.m_annotations;
         m_squareAnnotations = game.m_squareAnnotations;
         m_arrowAnnotations = game.m_arrowAnnotations;
-        m_clkAnnotations = game.m_clkAnnotations;
-        m_egtAnnotations = game.m_egtAnnotations;
-
         m_currentNode = game.m_currentNode;
         m_startPly = game.m_startPly;
         m_currentBoard = game.m_currentBoard;
 
         m_moveNodes = game.m_moveNodes;
-
-        setModified(true);
     }
     return *this;
 }
@@ -58,7 +51,7 @@ Game::~Game()
 {
 }
 
-MoveId Game::addMove(const Move& move, const QString& annotation, NagSet nags)
+MoveId Game::dbAddMove(const Move& move, const QString& annotation, NagSet nags)
 {
     MoveNode node;
     MoveId previousNode = m_currentNode;
@@ -68,20 +61,40 @@ MoveId Game::addMove(const Move& move, const QString& annotation, NagSet nags)
     node.parentNode = m_moveNodes[m_currentNode].parentNode;
     node.move = move;
     node.nags = nags;
-    node.ply = ply() + 1;
+    node.SetPly(ply() + 1);
     m_moveNodes.append(node);
     m_currentNode = m_moveNodes.size() - 1;
     if(!annotation.isEmpty())
     {
-        setAnnotation(annotation);
+        dbSetAnnotation(annotation);
     }
     m_moveNodes[previousNode].nextNode = m_currentNode;
     m_currentBoard.doMove(move);
-    setModified(true);
 
     return m_currentNode;
 }
 
+MoveId Game::addMove(const Move& move, const QString& annotation, NagSet nags)
+{
+    Game state = *this;
+    dbAddMove(move,annotation,nags);
+    indicateAnnotationsOnBoard(m_currentNode);
+    emit signalGameModified(true,state,tr("Add move"));
+    return m_currentNode;
+}
+
+MoveId Game::dbAddMove(const QString& sanMove, const QString& annotation, NagSet nags)
+{
+    Move move = m_currentBoard.parseMove(sanMove);
+    if(move.isLegal() || move.isNullMove())
+    {
+        return dbAddMove(move, annotation, nags);
+    }
+    // qDebug() << sanMove << " : " << move.toAlgebraicDebug() << " is illegal in position " << board().toHumanFen() << " / " << board().toFen();
+    return NO_MOVE;
+}
+
+
 MoveId Game::addMove(const QString& sanMove, const QString& annotation, NagSet nags)
 {
     Move move = m_currentBoard.parseMove(sanMove);
@@ -102,7 +115,7 @@ bool Game::mergeNode(Game& otherGame)
     otherGame.forward();
     QString san = otherGame.moveToSan(MoveOnly, PreviousMove, CURRENT_MOVE, &ann, &nags);
 
-    bool retVal = (NO_MOVE != addMove(san, ann, nags));
+    bool retVal = (NO_MOVE != dbAddMove(san, ann, nags));
     return retVal;
 }
 
@@ -112,7 +125,7 @@ bool Game::mergeVariations(Game& otherGame)
     bool ok = true;
     if(variationList.size())
     {
-        int otherCurrent = otherGame.currentMove();
+        MoveId otherCurrent = otherGame.currentMove();
         for(QList<MoveId>::iterator iter = variationList.begin(); iter != variationList.end(); ++iter)
         {
             otherGame.enterVariation(*iter);
@@ -139,7 +152,7 @@ bool Game::mergeAsMainline(Game& otherGame)
 
     QString san = otherGame.moveToSan(MoveOnly, PreviousMove, CURRENT_MOVE, &ann, &nags);
 
-    if(NO_MOVE != addMove(san, ann, nags))
+    if(NO_MOVE != dbAddMove(san, ann, nags))
     {
         while(!otherGame.atLineEnd())
         {
@@ -168,7 +181,7 @@ bool Game::mergeAsVariation(Game& otherGame)
 
     QString san = otherGame.moveToSan(MoveOnly, PreviousMove, CURRENT_MOVE, &ann, &nags);
 
-    if(NO_MOVE != addVariation(san, ann, nags))
+    if(NO_MOVE != dbAddVariation(san, ann, nags))
     {
         while(!otherGame.atLineEnd())
         {
@@ -236,6 +249,7 @@ MoveId Game::findMergePoint(const Game& otherGame)
 
 void Game::mergeWithGame(const Game& g)
 {
+    Game state = *this;
     MoveId saveNode = m_currentNode;
     Game otherGame = g;
 
@@ -246,7 +260,7 @@ void Game::mergeWithGame(const Game& g)
     QString black = otherGame.tag(TagNameBlack);
     QString event = otherGame.eventInfo();
     QString shortDescription = QString("%1-%2 %3").arg(white).arg(black).arg(event);
-    otherGame.setAnnotation(shortDescription);
+    otherGame.dbSetAnnotation(shortDescription);
 
     MoveId otherMergeNode = findMergePoint(otherGame);
 
@@ -276,6 +290,8 @@ void Game::mergeWithGame(const Game& g)
         moveToId(saveNode);
     }
     compact();
+
+    emit signalGameModified(true,state,tr("Merge game"));
 }
 
 QString Game::eventInfo() const
@@ -333,24 +349,31 @@ bool Game::currentNodeHasMove(Square from, Square  to) const
     {
         return true;
     }
-    int node = m_moveNodes[m_currentNode].nextNode;
+    MoveId node = m_moveNodes[m_currentNode].nextNode;
     if(node == NO_MOVE)
     {
         return true;
     }
     Move m = m_moveNodes[node].move;
-    if(m.from() == from && m.to() == to)
-    {
-        return (m_moveNodes[node].nextNode != NO_MOVE);
-    }
-    return false;
+    return (m.from() == from && m.to() == to);
+}
+
+bool Game::hasNextMove() const
+{
+    MoveId node = m_moveNodes[m_currentNode].nextNode;
+    return (node != NO_MOVE);
+}
+
+bool Game::findNextMove(Move m)
+{
+    return findNextMove(m.from(),m.to(),m.isPromotion() ? pieceType(m.promotedPiece()) : None);
 }
 
 // does the next main move or one of the variations go from square from to square to
 // if so make it on the board
 bool Game::findNextMove(Square from, Square to, PieceType promotionPiece)
 {
-    int node;
+    MoveId node;
     node = m_moveNodes[m_currentNode].nextNode;
     if(node != NO_MOVE)
     {
@@ -383,19 +406,21 @@ bool Game::findNextMove(Square from, Square to, PieceType promotionPiece)
 
 bool Game::replaceMove(const Move& move, const QString& annotation, NagSet nags, bool bReplace)
 {
-    int node;
+    MoveId node;
+    Game state = *this;
+
     node = m_moveNodes[m_currentNode].nextNode;
-    setModified(true);
     if(node == NO_MOVE)
     {
-        addMove(move, annotation, nags);
+        dbAddMove(move, annotation, nags);
+        emit signalGameModified(true,state,tr("Add move"));
         return true;
     }
 
     //replace node data with new move
     m_moveNodes[node].move = move;
     m_moveNodes[node].nags = nags;
-    setAnnotation(annotation, node);
+    dbSetAnnotation(annotation, node);
 
     //remove any following nodes after replaced move by disconnecting them from the tree
     forward();
@@ -406,9 +431,10 @@ bool Game::replaceMove(const Move& move, const QString& annotation, NagSet nags,
     else
     {
         truncateVariationAfterNextIllegalPosition();
-
+        compact();
     }
     backward();
+    emit signalGameModified(true,state,tr("Replace move"));
 
     return true;
 }
@@ -424,10 +450,43 @@ bool Game::insertMove(Move m)
 }
 
 MoveId Game::addVariation(const Move& move, const QString& annotation, NagSet nags)
+{
+    Game state = *this;
+    MoveId retVal = dbAddVariation(move, annotation, nags);
+    if (retVal != NO_MOVE)
+    {
+        emit signalGameModified(true, state, tr("Add Variation"));
+    }
+    return retVal;
+}
+
+MoveId Game::addVariation(const MoveList& moveList, const QString& annotation)
+{
+    Game state = *this;
+    MoveId retVal = dbAddVariation(moveList, annotation);
+    if (retVal != NO_MOVE)
+    {
+        emit signalGameModified(true, state, tr("Add Variation"));
+    }
+    return retVal;
+}
+
+MoveId Game::addVariation(const QString& sanMove, const QString& annotation, NagSet nags)
+{
+    Game state = *this;
+    MoveId retVal = dbAddVariation(sanMove, annotation, nags);
+    if (retVal != NO_MOVE)
+    {
+        emit signalGameModified(true, state, tr("Add Variation"));
+    }
+    return retVal;
+}
+
+MoveId Game::dbAddVariation(const Move& move, const QString& annotation, NagSet nags)
 {
     MoveId previousNode = m_currentNode;
     MoveId saveNextNode = m_moveNodes[m_currentNode].nextNode;
-    MoveId node = addMove(move, annotation, nags);
+    MoveId node = dbAddMove(move, annotation, nags);
     m_moveNodes[m_currentNode].parentNode = previousNode;
     m_moveNodes[previousNode].variations.append(node);
     m_moveNodes[previousNode].nextNode = saveNextNode;
@@ -435,7 +494,7 @@ MoveId Game::addVariation(const Move& move, const QString& annotation, NagSet na
     return (m_moveNodes.size() - 1);
 }
 
-MoveId Game::addVariation(const MoveList& moveList, const QString& annotation)
+MoveId Game::dbAddVariation(const MoveList& moveList, const QString& annotation)
 {
     if(moveList.isEmpty())
     {
@@ -446,38 +505,38 @@ MoveId Game::addVariation(const MoveList& moveList, const QString& annotation)
     int start = 1;
     if(!atLineEnd())
     {
-        varStart = addVariation(moveList.first());
+        varStart = dbAddVariation(moveList.first());
     }
     else if(!atGameStart())
     {
         Move oldMove = move();
         backward();
-        varStart = addVariation(oldMove);
+        varStart = dbAddVariation(oldMove);
         start = 0;
     }
     else
     {
-        varStart = addMove(moveList.first());
+        varStart = dbAddMove(moveList.first());
     }
     moveToId(varStart);
     for(int i = start; i < moveList.count(); ++i)
     {
-        addMove(moveList[i]);
+        dbAddMove(moveList[i]);
         forward();
     }
     if(!annotation.isEmpty())
     {
-        setAnnotation(annotation);
+        dbSetAnnotation(annotation);
     }
     moveToId(currentPosition);
     return varStart;
 }
 
-MoveId Game::addVariation(const QString& sanMove, const QString& annotation, NagSet nags)
+MoveId Game::dbAddVariation(const QString& sanMove, const QString& annotation, NagSet nags)
 {
     MoveId previousNode = m_currentNode;
     MoveId saveNextNode = m_moveNodes[m_currentNode].nextNode;
-    MoveId node = addMove(sanMove, annotation, nags);
+    MoveId node = dbAddMove(sanMove, annotation, nags);
     if(node == NO_MOVE)
     {
         return node;
@@ -496,7 +555,7 @@ bool Game::promoteVariation(MoveId variation)
     {
         return false;
     }
-
+    Game state = *this;
     MoveId currentNode = m_currentNode;	// Save current move
 
     // Find first move of the variation
@@ -516,6 +575,7 @@ bool Game::promoteVariation(MoveId variation)
     qSwap(m_moveNodes[parent].nextNode, m_moveNodes[parent].variations[index]);
 
     moveToId(currentNode);	// Restore current move
+    emit signalGameModified(true, state, tr("Promote Variation"));
     return true;
 }
 
@@ -526,6 +586,7 @@ bool Game::removeVariation(MoveId variation)
     {
         return false;
     }
+    Game state = *this;
     MoveId parentNode = m_moveNodes[variation].parentNode;
     removeNode(variation);
     moveToId(parentNode);
@@ -534,6 +595,7 @@ bool Game::removeVariation(MoveId variation)
     vars.removeAt(vars.indexOf(variation));
 
     compact();
+    emit signalGameModified(true, state, tr("Remove variation"));
     return true;
 }
 
@@ -549,9 +611,9 @@ void Game::truncateVariationAfterNextIllegalPosition()
     }
     QString san = moveToSan();
     Game g = *this;
-    if(NO_MOVE == g.addMove(san))
+    if(NO_MOVE == g.dbAddMove(san))
     {
-        int node = m_moveNodes[m_currentNode].nextNode;
+        MoveId node = m_moveNodes[m_currentNode].nextNode;
         removeNode(node);
     }
     else
@@ -565,9 +627,10 @@ void Game::truncateVariationAfterNextIllegalPosition()
 
 void Game::truncateVariation(Position position)
 {
+    Game state = *this;
     if(position == AfterMove)
     {
-        int node = m_moveNodes[m_currentNode].nextNode;
+        MoveId node = m_moveNodes[m_currentNode].nextNode;
         removeNode(node);
     }
     else if(position == BeforeMove && m_currentNode != 0)
@@ -580,7 +643,7 @@ void Game::truncateVariation(Position position)
         MoveId current = m_currentNode;
         MoveNode firstNode;
         firstNode.nextNode = m_currentNode;
-        firstNode.ply = m_moveNodes[m_currentNode].ply - 1;
+        firstNode.SetPly(m_moveNodes[m_currentNode].Ply() - 1);
         // Keep variation if truncating main line
         if(m_moveNodes[m_moveNodes[m_currentNode].previousNode].nextNode == m_currentNode)
         {
@@ -603,6 +666,7 @@ void Game::truncateVariation(Position position)
         moveToId(current);
     }
     compact();
+    emit signalGameModified(true, state, tr("Truncate variation"));
 }
 
 const Board& Game::board() const
@@ -637,28 +701,31 @@ void Game::setGameComment(const QString& gameComment)
 
 void Game::removeVariations()
 {
+    Game state = *this;
     for(int i = 0; i < m_moveNodes.size(); ++i)
     {
-        m_moveNodes[i].variations.clear();
+        while (!m_moveNodes[i].variations.empty())
+        {
+            removeVariation(m_moveNodes[i].variations.at(0));
+        }
     }
     compact();
-    setModified(true);
+    emit signalGameModified(true, state, tr("Remove variations"));
 }
 
 void Game::removeComments()
 {
+    Game state = *this;
     m_variationStartAnnotations.clear();
     m_annotations.clear();
     m_squareAnnotations.clear();
     m_arrowAnnotations.clear();
-    m_clkAnnotations.clear();
-    m_egtAnnotations.clear();
     for(int i = 0; i < m_moveNodes.size(); ++i)
     {
         m_moveNodes[i].nags.clear();
     }
     compact();
-    setModified(true);
+    emit signalGameModified(true, state, tr("Remove comments"));
 }
 
 bool Game::isMainline(MoveId moveId) const
@@ -680,15 +747,15 @@ bool Game::isMainline(MoveId moveId) const
 
 Result Game::result() const
 {
-    if(m_tags["Result"] == "1-0")
+    if(m_tags[TagNameResult] == "1-0")
     {
         return WhiteWin;
     }
-    else if(m_tags["Result"] == "1/2-1/2")
+    else if(m_tags[TagNameResult] == "1/2-1/2")
     {
         return Draw;
     }
-    else if(m_tags["Result"] == "0-1")
+    else if(m_tags[TagNameResult] == "0-1")
     {
         return BlackWin;
     }
@@ -698,6 +765,26 @@ Result Game::result() const
     }
 }
 
+int Game::resultAsInt() const
+{
+    if(m_tags[TagNameResult] == "1-0")
+    {
+        return +1;
+    }
+    else if(m_tags[TagNameResult] == "1/2-1/2")
+    {
+        return 0;
+    }
+    else if(m_tags[TagNameResult] == "0-1")
+    {
+        return -1;
+    }
+    else
+    {
+        return 0;
+    }
+}
+
 int Game::numberOfSiblings(MoveId moveId) const
 {
     MoveId node = nodeValid(moveId);
@@ -749,6 +836,17 @@ bool Game::atLineEnd(MoveId moveId) const
 }
 
 bool Game::setAnnotation(QString annotation, MoveId moveId, Position position)
+{
+    Game state = *this;
+    if (dbSetAnnotation(annotation, moveId, position))
+    {
+        emit signalGameModified(true, state, tr("Set annotation"));
+        return true;
+    }
+    return false;
+}
+
+bool Game::dbSetAnnotation(QString annotation, MoveId moveId, Position position)
 {
     MoveId node = nodeValid(moveId);
     if(node == NO_MOVE)
@@ -756,8 +854,6 @@ bool Game::setAnnotation(QString annotation, MoveId moveId, Position position)
         return false;
     }
 
-    setModified(true);
-
     QString specAnnot = specialAnnotation(annotation, "[%csl");
     if(position == Game::AfterMove || node == 0)
     {
@@ -778,30 +874,9 @@ bool Game::setAnnotation(QString annotation, MoveId moveId, Position position)
         setArrowAnnotation(specAnnot, m_moveNodes[node].previousNode);
     }
 
-    specAnnot = specialAnnotation(annotation, "[%egt");
-    if(position == Game::AfterMove || node == 0)
-    {
-        setEgtAnnotation(specAnnot, node);
-    }
-    else
-    {
-        setEgtAnnotation(specAnnot, m_moveNodes[node].previousNode);
-    }
-
-    specAnnot = specialAnnotation(annotation, "[%clk");
-    if(position == Game::AfterMove || node == 0)
-    {
-        setClkAnnotation(specAnnot, node);
-    }
-    else
-    {
-        setClkAnnotation(specAnnot, m_moveNodes[node].previousNode);
-    }
-
     annotation.remove(QRegExp("\\[%csl[^\\]]*\\]"));
     annotation.remove(QRegExp("\\[%cal[^\\]]*\\]"));
-    annotation.remove(QRegExp("\\[%clk[^\\]]*\\]"));
-    annotation.remove(QRegExp("\\[%egt[^\\]]*\\]"));
+
     int moves;
     int comments;
     int nags;
@@ -820,13 +895,20 @@ bool Game::setAnnotation(QString annotation, MoveId moveId, Position position)
     }
     else if(canHaveStartAnnotation(node))  	// Pre-move comment
     {
-        if(annotation.isEmpty())
+        if (node == 0)
         {
-            m_variationStartAnnotations.remove(node);
+            setGameComment(annotation);
         }
         else
         {
-            m_variationStartAnnotations[node] = annotation;
+            if(annotation.isEmpty() && (node > 1)) // Do not remove empty comment
+            {
+                m_variationStartAnnotations.remove(node);
+            }
+            else
+            {
+                m_variationStartAnnotations[node] = annotation;
+            }
         }
     }
     return true;
@@ -853,7 +935,7 @@ bool Game::setSquareAnnotation(QString squareAnnotation, MoveId moveId)
 
 bool Game::appendSquareAnnotation(Square s, QChar colorCode)
 {
-    setModified(true);
+    Game state = *this;
     QString newAnnot;
     QString annot = squareAnnotation();
     QString oldAnnot = annot;
@@ -896,6 +978,8 @@ bool Game::appendSquareAnnotation(Square s, QChar colorCode)
         }
     }
     setSquareAnnotation(newAnnot);
+    indicateAnnotationsOnBoard(currentMove());
+    emit signalGameModified(true, state, tr("Colorize square"));
     return true;
 }
 
@@ -905,7 +989,7 @@ bool Game::appendArrowAnnotation(Square dest, Square src, QChar colorCode)
     {
         return false;
     }
-    setModified(true);
+    Game state = *this;
     QString newAnnot;
     QString annot = arrowAnnotation();
     QString sqSrc = strSquareNames[src];
@@ -919,7 +1003,6 @@ bool Game::appendArrowAnnotation(Square dest, Square src, QChar colorCode)
     }
     else
     {
-
         annot.replace(QRegExp(QString(",.") + sqSrc + sqDest), ""); 	// if not first annotation remove annotation with preceding comma
         annot.replace(QRegExp(QString(".") + sqSrc + sqDest + ","), ""); // if first annotation remove annotation with trailing comma
         annot.replace(QRegExp(QString(".") + sqSrc + sqDest), ""); 	// if only annotation remove annotation
@@ -943,6 +1026,8 @@ bool Game::appendArrowAnnotation(Square dest, Square src, QChar colorCode)
         }
     }
     setArrowAnnotation(newAnnot);
+    indicateAnnotationsOnBoard(currentMove());
+    emit signalGameModified(true, state, tr("Paint arrow"));
     return true;
 }
 
@@ -997,25 +1082,6 @@ QString Game::arrowAnnotation(MoveId moveId) const
     return m_arrowAnnotations[node].simplified();
 }
 
-bool Game::setEgtAnnotation(QString annotation, MoveId moveId)
-{
-    MoveId node = nodeValid(moveId);
-    if(node == NO_MOVE)
-    {
-        return false;
-    }
-
-    if(annotation.isEmpty())
-    {
-        m_egtAnnotations.remove(node);
-    }
-    else
-    {
-        m_egtAnnotations[node] = annotation;
-    }
-    return true;
-}
-
 QString Game::egtAnnotation(MoveId moveId) const
 {
     MoveId node = nodeValid(moveId);
@@ -1024,43 +1090,61 @@ QString Game::egtAnnotation(MoveId moveId) const
         return "";
     }
 
-    QString annotation = m_egtAnnotations[node];
+    QString annotation = m_annotations[node];
     if(annotation.isNull())
     {
         return "";
     }
-    return m_egtAnnotations[node];
+
+    QRegExp egt("\\[%egt\\s*(\\d:\\d\\d:\\d\\d)\\]");
+    int pos = egt.indexIn(annotation);
+    if(pos >= 0)
+    {
+        return egt.cap(1);
+    }
+
+    return "";
 }
 
+QString Game::emtAnnotation(MoveId moveId) const
+{
+    MoveId node = nodeValid(moveId);
+    if(node == NO_MOVE)
+    {
+        return "";
+    }
+
+    QString annotation = m_annotations[node];
+    if(annotation.isNull())
+    {
+        return "";
+    }
+
+    QRegExp emt("\\[%emt\\s*(\\d:\\d\\d:\\d\\d)\\]");
+    int pos = emt.indexIn(annotation);
+    if(pos >= 0)
+    {
+        return emt.cap(1);
+    }
+
+    return "";
+}
+
+
 QString Game::timeAnnotation(MoveId moveId) const
 {
     QString s = clkAnnotation(moveId);
     if(s.isEmpty())
     {
-        return egtAnnotation(moveId);
+        s = egtAnnotation(moveId);
+    }
+    if (s.isEmpty())
+    {
+        s = emtAnnotation(moveId);
     }
     return s;
 }
 
-bool Game::setClkAnnotation(QString annotation, MoveId moveId)
-{
-    MoveId node = nodeValid(moveId);
-    if(node == NO_MOVE)
-    {
-        return false;
-    }
-
-    if(annotation.isEmpty())
-    {
-        m_clkAnnotations.remove(node);
-    }
-    else
-    {
-        m_clkAnnotations[node] = annotation;
-    }
-    return true;
-}
-
 QString Game::clkAnnotation(MoveId moveId) const
 {
     MoveId node = nodeValid(moveId);
@@ -1069,12 +1153,20 @@ QString Game::clkAnnotation(MoveId moveId) const
         return "";
     }
 
-    QString annotation = m_clkAnnotations[node];
+    QString annotation = m_annotations[node];
     if(annotation.isNull())
     {
         return "";
     }
-    return m_clkAnnotations[node];
+
+    QRegExp clk("\\[%clk\\s*(\\d:\\d\\d:\\d\\d)\\]");
+    int pos = clk.indexIn(annotation);
+    if(pos >= 0)
+    {
+        return clk.cap(1);
+    }
+
+    return "";
 }
 
 QString Game::composeAnnotation(QString freeText, MoveId node) const
@@ -1087,14 +1179,6 @@ QString Game::composeAnnotation(QString freeText, MoveId node) const
     {
         freeText.append(QString("[%cal %1]").arg(m_arrowAnnotations[node]));
     }
-    if(!m_clkAnnotations[node].isEmpty())
-    {
-        freeText.append(QString("[%clk %1]").arg(m_clkAnnotations[node]));
-    }
-    if(!m_egtAnnotations[node].isEmpty())
-    {
-        freeText.append(QString("[%egt %1]").arg(m_egtAnnotations[node]));
-    }
     return freeText;
 }
 
@@ -1120,22 +1204,30 @@ QString Game::annotation(MoveId moveId, Position position) const
 
 QString Game::textAnnotation(MoveId moveId, Position position) const
 {
+    QString s;
     MoveId node = nodeValid(moveId);
     if(position == AfterMove)
     {
-        return m_annotations.value(node, "");
+        s = m_annotations.value(node, "");
     }
     else
     {
         if(node == 0)
         {
-            return m_annotations.value(0, "");
+            s = m_annotations.value(0, "");
         }
         else
         {
-            return m_variationStartAnnotations.value(node, "");
+            s = m_variationStartAnnotations.value(node, "");
         }
     }
+    QRegExp emt("\\[%emt\\s*(\\d:\\d\\d:\\d\\d)\\]");
+    s.remove(emt);
+    QRegExp egt("\\[%egt\\s*(\\d:\\d\\d:\\d\\d)\\]");
+    s.remove(egt);
+    QRegExp clk("\\[%clk\\s*(\\d:\\d\\d:\\d\\d)\\]");
+    s.remove(clk);
+    return s;
 }
 
 bool Game::canHaveStartAnnotation(MoveId moveId) const
@@ -1149,8 +1241,9 @@ bool Game::addNag(Nag nag, MoveId moveId)
     MoveId node = nodeValid(moveId);
     if(node != NO_MOVE)
     {
+        Game state = *this;
         m_moveNodes[node].nags.addNag(nag);
-        setModified(true);
+        emit signalGameModified(true, state, tr("Add nag"));
         return true;
     }
     return false;
@@ -1161,8 +1254,9 @@ bool Game::setNags(NagSet nags, MoveId moveId)
     MoveId node = nodeValid(moveId);
     if(node != NO_MOVE)
     {
+        Game state = *this;
         m_moveNodes[node].nags = nags;
-        setModified(true);
+        emit signalGameModified(true, state, tr("Set nags"));
         return true;
     }
     return false;
@@ -1191,7 +1285,7 @@ MoveId Game::nodeValid(MoveId moveId) const
     }
     if((moveId >= 0) && (moveId < m_moveNodes.size()))
     {
-        if(m_moveNodes[moveId].removed)
+        if(m_moveNodes[moveId].Removed())
         {
             return NO_MOVE;
         }
@@ -1202,15 +1296,19 @@ MoveId Game::nodeValid(MoveId moveId) const
 
 void Game::moveCount(int* moves, int* comments, int* nags) const
 {
-    *moves = *comments = *nags = 0;
+    *moves = *comments = 0;
+    if (nags) *nags = 0;
 
-    int node = 1;
+    MoveId node = 1;
     while(nodeValid(node) != NO_MOVE)
     {
         *moves += 1;
-        if(m_moveNodes[node].nags.count() != 0)
+        if (nags)
         {
-            *nags += 1;
+            if(m_moveNodes[node].nags.count() != 0)
+            {
+                *nags += 1;
+            }
         }
         node = m_moveNodes[node].nextNode;
     }
@@ -1225,12 +1323,21 @@ void Game::moveCount(int* moves, int* comments, int* nags) const
     }
 }
 
+bool Game::isEmpty() const
+{
+    int moves;
+    int comments;
+    moveCount(&moves, &comments);
+    bool gameIsEmpty = ((moves+comments) == 0);
+    return gameIsEmpty;
+}
+
 int Game::ply(MoveId moveId) const
 {
     MoveId node = nodeValid(moveId);
     if(node != NO_MOVE)
     {
-        return m_moveNodes[node].ply;
+        return m_moveNodes[node].Ply();
     }
     return 0;
 }
@@ -1252,7 +1359,7 @@ int Game::moveNumber(MoveId moveId) const
 int Game::plyCount() const
 {
     int count = 0;
-    int node = 0;
+    MoveId node = 0;
 
     while(node != NO_MOVE)
     {
@@ -1293,8 +1400,9 @@ void Game::moveVariationUp(MoveId moveId)
     int i = v.indexOf(variation);
     if(i > 0)
     {
+        Game state = *this;
         v.swap(i, i - 1);
-        setModified(true);
+        emit signalGameModified(true, state, tr("Move variation"));
     }
 }
 
@@ -1327,8 +1435,9 @@ void Game::moveVariationDown(MoveId moveId)
     int i = v.indexOf(variation);
     if(i >= 0 && (i + 1) < v.count())
     {
+        Game state = *this;
         v.swap(i, i + 1);
-        setModified(true);
+        emit signalGameModified(true, state, tr("Move variation"));
     }
 }
 
@@ -1342,14 +1451,19 @@ void Game::enumerateVariations(MoveId moveId, char a)
     MoveId node = nodeValid(moveId);
     if(node != NO_MOVE)
     {
+        Game state = *this;
         MoveId parentNode = m_moveNodes[node].parentNode;
         QList <MoveId>& v = m_moveNodes[parentNode].variations;
-        for(int i = 0; i < v.size(); ++i)
+        if (v.size())
         {
-            QString oldAnnotation = annotation(v[i], Game::BeforeMove);
-            oldAnnotation.remove(QRegExp("^.\\)"));
-            QString s = QString("%1) %2").arg(QChar(a + i)).arg(oldAnnotation).trimmed();
-            setAnnotation(s, v[i], Game::BeforeMove);
+            for(int i = 0; i < v.size(); ++i)
+            {
+                QString oldAnnotation = annotation(v[i], Game::BeforeMove);
+                oldAnnotation.remove(QRegExp("^.\\)"));
+                QString s = QString("%1) %2").arg(QChar(a + i)).arg(oldAnnotation).trimmed();
+                dbSetAnnotation(s, v[i], Game::BeforeMove);
+            }
+            emit signalGameModified(true, state, tr("Enumerate variations"));
         }
     }
 }
@@ -1441,16 +1555,6 @@ bool Game::variationHasSiblings(MoveId variation) const
     return (variationCount(parent) > 1);
 }
 
-bool Game::isModified() const
-{
-    return m_isModified;
-}
-
-void Game::setModified(bool set)
-{
-    m_isModified = set;
-}
-
 void Game::indicateAnnotationsOnBoard(MoveId moveId)
 {
     QString annotation = squareAnnotation(moveId);
@@ -1458,6 +1562,8 @@ void Game::indicateAnnotationsOnBoard(MoveId moveId)
 
     annotation = arrowAnnotation(moveId);
     m_currentBoard.setArrowAnnotation(annotation);
+
+    emit signalMoveChanged();
 }
 
 void Game::moveToStart()
@@ -1488,7 +1594,7 @@ void Game::moveToId(MoveId moveId)
     }
 
     //jump to node, travelling back to start adding the moves to the stack
-    int node = moveId;
+    MoveId node = moveId;
     QStack < Move > moveStack;
     while(node)
     {
@@ -1507,27 +1613,6 @@ void Game::moveToId(MoveId moveId)
     indicateAnnotationsOnBoard(moveId);
 }
 
-int Game::moveToPly(int ply)
-{
-    if(ply == 0)
-    {
-        moveToStart();
-        return 0;
-    }
-
-    int currentPly = this->ply();
-    int diff = ply - currentPly;
-
-    if(diff > 0)
-    {
-        return currentPly + forward(diff);
-    }
-    else
-    {
-        return currentPly - backward(-diff);
-    }
-}
-
 Move Game::move(MoveId moveId) const
 {
     MoveId node = nodeValid(moveId);
@@ -1603,7 +1688,10 @@ void Game::removeNode(MoveId moveId)
     MoveId node = nodeValid(moveId);
     if(node != NO_MOVE)
     {
-        setModified(true);
+        m_variationStartAnnotations.remove(node);
+        m_annotations.remove(node);
+        m_squareAnnotations.remove(node);
+        m_arrowAnnotations.remove(node);
 
         if(variationCount(node))
         {
@@ -1628,8 +1716,6 @@ void Game::clear()
     m_annotations.clear();
     m_squareAnnotations.clear();
     m_arrowAnnotations.clear();
-    m_clkAnnotations.clear();
-    m_egtAnnotations.clear();
 
     m_startPly = 0;
     m_currentNode = 0;
@@ -1638,13 +1724,11 @@ void Game::clear()
     m_currentBoard = m_startingBoard;
 
     m_moveNodes.append(MoveNode());
-    setModified(true);
 }
 
 void Game::clearTags()
 {
     m_tags.clear();
-    setModified(true);
 }
 
 QString Game::tag(const QString& tag) const
@@ -1669,10 +1753,17 @@ void Game::removeTag(const QString& tag)
 
 void Game::setStartingBoard(const Board& startingBoard)
 {
-    setStartingBoard(startingBoard.toFen());
+    Game state = *this;
+    dbSetStartingBoard(startingBoard.toFen());
+    emit signalGameModified(true, state, tr("Set starting board"));
 }
 
-void Game::setStartingBoard(const QString& fen)
+void Game::dbSetStartingBoard(const Board& startingBoard)
+{
+    dbSetStartingBoard(startingBoard.toFen());
+}
+
+void Game::dbSetStartingBoard(const QString& fen)
 {
     clear();
     m_startingBoard.fromFen(fen);
@@ -1687,8 +1778,9 @@ void Game::setStartingBoard(const QString& fen)
 
 void Game::setResult(Result result)
 {
+    Game state = *this;
     m_tags["Result"] = resultString(result);
-    setModified(true);
+    emit signalGameModified(true, state, tr("Set result"));
 }
 
 QString Game::moveToSan(MoveStringFlags flags, NextPreviousMove nextPrevious, MoveId moveId,
@@ -1756,7 +1848,7 @@ QString Game::moveToSan(MoveStringFlags flags, NextPreviousMove nextPrevious, Mo
     return san;
 }
 
-void Game::dumpMoveNode(MoveId moveId)
+void Game::dumpMoveNode(MoveId moveId) const
 {
     if(moveId == CURRENT_MOVE)
     {
@@ -1765,18 +1857,21 @@ void Game::dumpMoveNode(MoveId moveId)
     if(moveId != NO_MOVE)
     {
         qDebug() << "Move Id : " << moveId;
-        qDebug() << "   Next node   : " << m_moveNodes[moveId].nextNode;
-        qDebug() << "   Prev node   : " << m_moveNodes[moveId].previousNode;
-        qDebug() << "   Parent node : " << m_moveNodes[moveId].parentNode;
-        qDebug() << "   Nags        : " << m_moveNodes[moveId].nags.toString(NagSet::Simple);
-        qDebug() << "   Deleted     : " << m_moveNodes[moveId].removed;
-        qDebug() << "   # Variations: " << m_moveNodes[moveId].variations.size();
-        qDebug() << "   Variations  : " << m_moveNodes[moveId].variations;
-        qDebug() << "   Move        : " << moveToSan(FullDetail, PreviousMove, moveId);
+        qDebug() << "   Next node   : " << m_moveNodes.at(moveId).nextNode;
+        qDebug() << "   Prev node   : " << m_moveNodes.at(moveId).previousNode;
+        qDebug() << "   Parent node : " << m_moveNodes.at(moveId).parentNode;
+        qDebug() << "   Nags        : " << m_moveNodes.at(moveId).nags.toString(NagSet::Simple);
+        qDebug() << "   Deleted     : " << m_moveNodes.at(moveId).Removed();
+        qDebug() << "   # Variations: " << m_moveNodes.at(moveId).variations.size();
+        qDebug() << "   Variations  : " << m_moveNodes.at(moveId).variations;
+        qDebug() << "   Move        : " << m_moveNodes.at(moveId).move.toAlgebraic()
+                 << " (" << m_moveNodes.at(moveId).move.rawMove()
+                 << ", " << m_moveNodes.at(moveId).move.rawUndo()
+                 << ")";
     }
 }
 
-void Game::dumpAnnotations(MoveId moveId)
+void Game::dumpAnnotations(MoveId moveId) const
 {
     if(moveId == CURRENT_MOVE)
     {
@@ -1784,18 +1879,16 @@ void Game::dumpAnnotations(MoveId moveId)
     }
     if(moveId != NO_MOVE)
     {
-        qDebug() << "   Annotations : ";
-        qDebug() << "   Text        : " << m_annotations[moveId];
-        qDebug() << "   Square      : " << m_squareAnnotations[moveId];
-        qDebug() << "   Arrow       : " << m_arrowAnnotations[moveId];
-        qDebug() << "   CLK         : " << m_clkAnnotations[moveId];
-        qDebug() << "   EGT         : " << m_egtAnnotations[moveId];
-        qDebug() << "   Start       : " << m_variationStartAnnotations[moveId];
+        qDebug() << "   Annotations";
+        qDebug() << "     Text        : " << m_annotations.value(moveId);
+        qDebug() << "     Square      : " << m_squareAnnotations.value(moveId);
+        qDebug() << "     Arrow       : " << m_arrowAnnotations.value(moveId);
+        qDebug() << "     Start       : " << m_variationStartAnnotations.value(moveId);
     }
 }
 
 
-void Game::dumpAllMoveNodes()
+void Game::dumpAllMoveNodes() const
 {
     qDebug() << endl;
     qDebug() << "Current Node: " << m_currentNode;
@@ -1806,10 +1899,10 @@ void Game::dumpAllMoveNodes()
     }
     int moves, comments, nags;
     moveCount(&moves, &comments, &nags);
-    qDebug() << "Moves: " << moves << " Comments: " << comments << " Nags: " << nags;
+    qDebug() << "Moves: " << moves << " Comments: " << comments << " Nags: " << nags << endl;
 }
 
-int Game::findPosition(const Board& position) const
+MoveId Game::findPosition(const Board& position) const
 {
     MoveId current = 0;
     Board currentBoard(m_startingBoard);
@@ -1853,20 +1946,54 @@ bool Game::loadEcoFile(const QString& ecoFile)
 void Game::compact()
 {
     int oldSize = m_moveNodes.size();
-    QList <MoveNode> moveNodes;
-    QVector <int> oldIdNewIdMapping(oldSize, NO_MOVE);
+    QList<MoveNode> moveNodes;
+    QMap<MoveId,MoveId> oldIdNewIdMapping;
+    QList<MoveId> removedNodes;
 
-    for(int i = 0; i < oldSize; ++i)
+    for(MoveId i = 0; i < oldSize; ++i)
     {
-        if(!m_moveNodes[i].removed)
+        if(!m_moveNodes[i].Removed())
         {
             oldIdNewIdMapping[i] = moveNodes.size();
             moveNodes.append(m_moveNodes[i]);
         }
+        else
+        {
+            removedNodes.push_back(i);
+        }
+    }
+
+    foreach(MoveId m, removedNodes)
+    {
+        m_variationStartAnnotations.remove(m);
+        m_annotations.remove(m);
+        m_squareAnnotations.remove(m);
+        m_arrowAnnotations.remove(m);
+    }
+
+    AnnotationMap variationStartAnnotations;
+    AnnotationMap annotations;
+    AnnotationMap squareAnnotations;
+    AnnotationMap arrowAnnotations;
+
+    foreach(MoveId key, oldIdNewIdMapping.keys())
+    {
+        MoveId n = oldIdNewIdMapping.value(key);
+#define GAME_UPDATE_ANNOT(t,x) \
+        if (x.contains(key)) \
+        {\
+            QString s = x.value(key);\
+            t[n] = s;\
+        }
+
+        GAME_UPDATE_ANNOT(variationStartAnnotations, m_variationStartAnnotations);
+        GAME_UPDATE_ANNOT(annotations, m_annotations);
+        GAME_UPDATE_ANNOT(squareAnnotations, m_squareAnnotations);
+        GAME_UPDATE_ANNOT(arrowAnnotations, m_arrowAnnotations);
     }
 
     // update nodes links to other nodes in shrinked list (prev, next, variations)
-    for(int i = 0, newSize = moveNodes.size(); i < newSize; ++i)
+    for(MoveId i = 0, newSize = moveNodes.size(); i < newSize; ++i)
     {
         MoveNode& node = moveNodes[i];
 #define GAME_UPDATE_MOVEID(aMoveId) if (aMoveId != NO_MOVE) aMoveId = oldIdNewIdMapping[aMoveId]
@@ -1879,31 +2006,39 @@ void Game::compact()
             GAME_UPDATE_MOVEID(vars[j]);
         }
         vars.removeAll(NO_MOVE);
-#undef  GAME_UPDATE_LINK
+#undef GAME_UPDATE_MOVEID
     }
 
+    m_variationStartAnnotations.clear();
+    m_annotations.clear();
+    m_squareAnnotations.clear();
+    m_arrowAnnotations.clear();
+
+    m_variationStartAnnotations = variationStartAnnotations;
+    m_annotations               = annotations;
+    m_squareAnnotations         = squareAnnotations;
+    m_arrowAnnotations          = arrowAnnotations;
+
     m_moveNodes.clear();
     m_moveNodes = moveNodes;
     m_currentNode = oldIdNewIdMapping[m_currentNode];
 }
 
-QString Game::ecoClassify()
+QString Game::ecoClassify() const
 {
     //move to end of main line
-    MoveId currentNode = m_currentNode;
-    moveToEnd();
+    Game g = *this;
+    g.moveToEnd();
 
     //search backwards for the first eco position
-    while(backward())
+    while(g.backward())
     {
-        quint64 key = m_currentBoard.getHashValue();
+        quint64 key = g.board().getHashValue();
         if(m_ecoPositions.contains(key))
         {
-            moveToId(currentNode);
             return m_ecoPositions[key];
         }
     }
-    moveToId(currentNode);
 
     return QString();
 }
@@ -1914,15 +2049,27 @@ bool Game::isEcoPosition() const
     return (m_ecoPositions.contains(key));
 }
 
+QString Game::findEcoName(QString eco)
+{
+    foreach(QString actualEco, m_ecoPositions.values())
+    {
+        if (actualEco.startsWith(eco))
+        {
+            QString opName = actualEco.section(" ",1);
+            return opName;
+        }
+    }
+    return QString();
+}
+
 void Game::reparentVariation(MoveId variation, MoveId parent)
 {
     if(variation != NO_MOVE)
     {
-        setModified(true);
-    }
-    for(MoveId node = variation; node != NO_MOVE; node = m_moveNodes[node].nextNode)
-    {
-        m_moveNodes[node].parentNode = parent;
+        for(MoveId node = variation; node != NO_MOVE; node = m_moveNodes[node].nextNode)
+        {
+            m_moveNodes[node].parentNode = parent;
+        }
     }
 }
 
@@ -1933,7 +2080,6 @@ QString Game::specialAnnotation(QString& annotation, QString specialMark) const
     // If we found a chessbase special annotation
     if(specialAnnotationStart >= 0)
     {
-
         int specialAnnotationDataStart = specialAnnotationStart + specialMark.length() + 1; //+ 1 for space after specialMark
         int specialAnnotationEnd = annotation.indexOf(']', specialAnnotationDataStart);
 
