@@ -21,7 +21,7 @@
 #define CURRENT_VARIATION -3
 #define COMPILED_ECO_FILE_ID ((quint32)0xCD5CBD02U)
 
-typedef int MoveId;
+typedef short MoveId;
 
 class SaveRestoreMove;
 class SaveRestoreMoveCompact;
@@ -113,6 +113,8 @@ public :
     /** @return annotation at move at node @p moveId. */
     QString egtAnnotation(MoveId moveId = CURRENT_MOVE) const;
     /** @return annotation at move at node @p moveId. */
+    QString emtAnnotation(MoveId moveId = CURRENT_MOVE) const;
+    /** @return annotation at move at node @p moveId. */
     QString clkAnnotation(MoveId moveId = CURRENT_MOVE) const;
     /** @return time annotation (either egt or clock) at move at node @p moveId. */
     QString timeAnnotation(MoveId moveId = CURRENT_MOVE) const;
@@ -132,6 +134,8 @@ public :
 
     // **** node modification methods ****
     /** Sets the comment associated with move at node @p moveId */
+    bool dbSetAnnotation(QString annotation, MoveId moveId = CURRENT_MOVE, Position position = AfterMove);
+    /** Sets the comment associated with move at node @p moveId */
     bool setAnnotation(QString annotation, MoveId moveId = CURRENT_MOVE, Position position = AfterMove);
 
     /** Sets the squareAnnotation associated with move at node @p moveId */
@@ -146,11 +150,6 @@ public :
     /** Sets the arrowAnnotation associated with move at node @p moveId */
     bool setArrowAnnotation(QString arrowAnnotation, MoveId moveId = CURRENT_MOVE);
 
-    /** Sets the annotation associated with move at node @p moveId */
-    bool setEgtAnnotation(QString annotation, MoveId moveId = CURRENT_MOVE);
-    /** Sets the annotation associated with move at node @p moveId */
-    bool setClkAnnotation(QString annotation, MoveId moveId = CURRENT_MOVE);
-
     /** Adds a nag to move at node @p moveId */
     bool addNag(Nag nag, MoveId moveId = CURRENT_MOVE);
     /** Sets nags for move at node @p moveId */
@@ -168,7 +167,9 @@ public :
     bool atLineEnd(MoveId moveId = CURRENT_MOVE) const;
     bool atGameEnd(MoveId moveId = CURRENT_MOVE) const;
     /** Counts the number of moves, comments and nags, in mainline, to the end of the game */
-    void moveCount(int* moves, int* comments, int* nags) const;
+    void moveCount(int* moves, int* comments, int* nags=0) const;
+    /** Determine if game contains something reasonable */
+    bool isEmpty() const;
     /** @return number of half moves made since the beginning of the game */
     int ply(MoveId moveId = CURRENT_MOVE) const;
     /** @return current move. Equals to (ply-1)/2+1 for standard games, but may be different
@@ -182,10 +183,6 @@ public :
     int variationCount(MoveId moveId = CURRENT_MOVE) const;
     /** @return true if the referenced variation has siblings */
     bool variationHasSiblings(MoveId variation) const;
-    /** @return true if the game has been modified */
-    bool isModified() const;
-    /** Clear/set game's @p modified flag. */
-    void setModified(bool set);
     /** @return moveId of the top main line */
     MoveId mainLineMove() const;
     /** @return moveId of the previous move */
@@ -206,14 +203,14 @@ public :
     int moveByPly(int diff);
     /** Moves to the position corresponding to the given move id */
     void moveToId(MoveId moveId);
-    /** Moves to the given ply */
-    int moveToPly(int ply);
     /** Move forward the given number of moves, returns actual number of moves made */
     int forward(int count = 1);
     /** Move back the given number of moves, returns actual number of moves undone */
     int backward(int count = 1);
     /** Moves forward if the next move matches (from,to,promotionPiece) */
     bool findNextMove(Square from, Square to, PieceType promotionPiece = None);
+    bool findNextMove(Move m);
+    bool hasNextMove() const;
 
     /** Enters the variation that corresponds to moveId. moveId must be a MoveId that
         corresponds to a subvariation of the current position. Compared to moveToId
@@ -223,8 +220,12 @@ public :
 
     // ***** game modification methods *****
     /** Adds a move at the current position, returns the move id of the added move */
+    MoveId dbAddMove(const Move& move, const QString& annotation = QString(), NagSet nags = NagSet());
+    /** Adds a move at the current position, returns the move id of the added move */
     MoveId addMove(const Move& move, const QString& annotation = QString(), NagSet nags = NagSet());
     /** Adds a move at the current position, returns the move id of the added move */
+    MoveId dbAddMove(const QString& sanMove, const QString& annotation = QString(), NagSet nags = NagSet());
+    /** Adds a move at the current position, returns the move id of the added move */
     MoveId addMove(const QString& sanMove, const QString& annotation = QString(), NagSet nags = NagSet());
     /** Replace the move after the current position */
     bool replaceMove(const Move& move, const QString& annotation = QString(), NagSet nags = NagSet(), bool bReplace = true);
@@ -241,6 +242,15 @@ public :
     /** Adds a move at the current position as a variation,
      * returns the move id of the added move */
     MoveId addVariation(const QString& sanMove, const QString& annotation = QString(), NagSet nags = NagSet());
+    /** Adds a move at the current position as a variation,
+     * returns the move id of the added move */
+    MoveId dbAddVariation(const Move& move, const QString& annotation = QString(), NagSet nags = NagSet());
+    /** Adds a move at the current position as a variation,
+     * returns the move id of the added move */
+    MoveId dbAddVariation(const MoveList& moveList, const QString& annotation = QString());
+    /** Adds a move at the current position as a variation,
+     * returns the move id of the added move */
+    MoveId dbAddVariation(const QString& sanMove, const QString& annotation = QString(), NagSet nags = NagSet());
     /** Merge current node of @p otherGame into this game */
     bool mergeNode(Game &otherGame);
     /** Merge @p otherGame starting from otherGames current position into this game as a new mainline */
@@ -266,9 +276,9 @@ public :
     /** Removes all tags and moves */
     void clear();
     /** Set the game start position */
-    void setStartingBoard(const Board& startingBoard);
+    void dbSetStartingBoard(const Board& startingBoard);
     /** Set the game start position from FEN. */
-    void setStartingBoard(const QString& fen);
+    void dbSetStartingBoard(const QString& fen);
     /** set comment associated with game */
     void setGameComment(const QString& gameComment);
     /** Remove all variations */
@@ -299,20 +309,21 @@ public :
     bool currentNodeHasVariation(Square from, Square to) const;
 
     /** @return ECO code for the game */
-    QString ecoClassify();
+    QString ecoClassify() const;
     /** @return true if current pos is in the ECO list */
     bool isEcoPosition() const;
 
     /** Method that loads a file containing ECO classifications for use by the ecoClassify method. Returns true if successful */
     static bool loadEcoFile(const QString& ecoFile);
+    static QString findEcoName(QString eco);
 
     /* Debug */
     /** Dump a move node using qDebug() */
-    void dumpMoveNode(MoveId moveId = CURRENT_MOVE);
+    void dumpMoveNode(MoveId moveId = CURRENT_MOVE) const;
     /** Dump annotatios for move @p moveId using qDebug() */
-    void dumpAnnotations(MoveId moveId);
+    void dumpAnnotations(MoveId moveId) const;
     /** Repeatedly call dumpMoveNode for all nodes */
-    void dumpAllMoveNodes();
+    void dumpAllMoveNodes() const;
     /** Decide if moveVariationUp() can be executed */
     bool canMoveVariationUp(MoveId moveId) const;
     /** Decide if moveVariationDown() can be executed */
@@ -324,69 +335,70 @@ public :
     /** Move the variation @p moveId of the curent node down in the list of variations */
     void moveVariationDown(MoveId moveId);
 
+    int resultAsInt() const;
+    void setStartingBoard(const Board &startingBoard);
 signals:
-    void signalGameModified();
+    void signalGameModified(bool,Game,QString);
+    void signalMoveChanged();
 
 private:
 
     QString specialAnnotation(QString& annotation, QString specialMark) const;  // [%csl  [%cal
     QString composeAnnotation(QString freeText, MoveId node) const;
 
+#pragma pack(push,2)
     struct MoveNode
     {
-        Move move;
-        NagSet nags;
         MoveId previousNode;
         MoveId nextNode;
         MoveId parentNode;
-        bool removed;
-        int ply;
-        QList <MoveId> variations;
+        short m_ply;
+        Move move;
+        NagSet nags;
+        QList<MoveId> variations;
         void remove()
         {
             parentNode = previousNode = nextNode = NO_MOVE;
-            removed = true;
+            m_ply |= 0x8000;
         }
         MoveNode()
         {
             parentNode = nextNode = previousNode = NO_MOVE;
-            removed = false;
-            ply = 0;
+            m_ply = 0;
         }
+        void SetPly(short ply) { Q_ASSERT(m_ply<0x7FFF); m_ply = ply; }
+        short Ply() const { return m_ply & 0x7FFF; }
+        bool Removed() const { return (m_ply & 0x8000); }
     };
+#pragma pack(pop)
 
     /** List of nodes */
     QList <MoveNode> m_moveNodes;
     /** Keeps the current node in the game */
     MoveId m_currentNode;
+    /** Keeps the start ply of the game, 0 for standard starting position */
+    short m_startPly;
     /** Keeps the start position of the game */
     Board m_startingBoard;
     /** Keeps the current position of the game */
     Board m_currentBoard;
-    /** Keeps the start ply of the game, 0 for standard starting position */
-    int m_startPly;
-    /** Flag indicating if the game has been modified */
-    bool m_isModified;
+
+    typedef QMap<MoveId, QString> AnnotationMap;
+
     /** Start annotations for each variation */
-    QMap <MoveId, QString> m_variationStartAnnotations;
+    AnnotationMap m_variationStartAnnotations;
     /** Annotations for move nodes */
-    QMap <MoveId, QString> m_annotations;
+    AnnotationMap m_annotations;
 
     /** SquareAnnotations for move nodes */
-    QMap <MoveId, QString> m_squareAnnotations;
+    AnnotationMap m_squareAnnotations;
 
     /** ArrowAnnotations for move nodes */
-    QMap <MoveId, QString> m_arrowAnnotations;
+    AnnotationMap m_arrowAnnotations;
 
     /** Map keeping pgn tags of the game */
     QMap<QString, QString> m_tags;
 
-    /** Remaining Time Annotations for move nodes */
-    QMap <MoveId, QString> m_clkAnnotations;
-
-    /** Elapsed Time Annotations for move nodes */
-    QMap <MoveId, QString> m_egtAnnotations;
-
     // **** memory  management methods ****
     /** Remove all removed nodes */
     void compact();
