@@ -25,6 +25,21 @@ MemoryDatabase::~MemoryDatabase()
     }
 }
 
+bool MemoryDatabase::isReadOnly() const
+{
+    return false;
+}
+
+bool MemoryDatabase::isModified() const
+{
+    return m_isModified;
+}
+
+void MemoryDatabase::setModified(bool b)
+{
+    m_isModified = b;
+}
+
 bool MemoryDatabase::appendGame(const Game& game)
 {
     // Add to index
@@ -42,27 +57,27 @@ bool MemoryDatabase::appendGame(const Game& game)
     newGame->clearTags();
     m_games.append(newGame);
     ++m_count;
-    m_isModified = true;
+    setModified(true);
     return true;
 }
 
-bool MemoryDatabase::remove(int gameId)
+bool MemoryDatabase::remove(GameId gameId)
 {
-    m_isModified = true;
+    setModified(true);
     m_index.setDeleted(gameId, true);
     return true;
 }
 
-bool MemoryDatabase::undelete(int gameId)
+bool MemoryDatabase::undelete(GameId gameId)
 {
-    m_isModified = true;
+    setModified(true);
     m_index.setDeleted(gameId, false);
     return true;
 }
 
-bool MemoryDatabase::replace(int index, Game& game)
+bool MemoryDatabase::replace(GameId gameId, Game& game)
 {
-    if(index >= m_count)
+    if(gameId >= m_count)
     {
         return false;
     }
@@ -71,35 +86,35 @@ bool MemoryDatabase::replace(int index, Game& game)
     QMap <QString, QString>::const_iterator i = tags.constBegin();
     while(i != tags.constEnd())
     {
-        m_index.setTag(i.key(), i.value(), index);
+        m_index.setTag(i.key(), i.value(), gameId);
         ++i;
     }
     // Upate game array
-    *m_games[index] = game;
-    m_games[index]->clearTags();
-    m_isModified = true;
+    *m_games[gameId] = game;
+    m_games[gameId]->clearTags();
+    setModified(true);
     return true;
 }
 
-void MemoryDatabase::loadGameMoves(int index, Game& game)
+void MemoryDatabase::loadGameMoves(GameId gameId, Game& game)
 {
-    if(index >= m_count)
+    if(gameId >= m_count)
     {
         return;
     }
-    game = *m_games[index];
+    game = *m_games[gameId];
 }
 
-bool MemoryDatabase::loadGame(int index, Game& game)
+bool MemoryDatabase::loadGame(GameId gameId, Game& game)
 {
-    if(index < 0 || index >= m_count || m_index.deleted(index))
+    if(gameId >= m_count || m_index.deleted(gameId))
     {
         return false;
     }
-    lock();
-    game = *m_games[index];
-    loadGameHeaders(index, game);
-    unlock();
+
+    game = *m_games[gameId];
+    loadGameHeaders(gameId, game);
+
     return true;
 }
 
@@ -109,7 +124,7 @@ void MemoryDatabase::parseGame()
     QString fen = m_index.tagValue(TagNameFEN, m_count - 1);
     if(fen != "?")
     {
-        game->setStartingBoard(fen);
+        game->dbSetStartingBoard(fen);
     }
     m_index.setValidFlag(m_count - 1, parseMoves(game));
     m_index.setTag("Length", QString::number((game->plyCount() + 1) / 2), m_count - 1);
@@ -129,7 +144,6 @@ void MemoryDatabase::parseGame()
             {
                 game->setTag("ECO", eco);
                 m_index.setTag("ECO", eco, m_count - 1);
-                m_isModified = true;
             }
         }
     }
@@ -140,17 +154,6 @@ void MemoryDatabase::parseGame()
 bool MemoryDatabase::parseFile()
 {
     bool ok = parseFileIntern();
-    m_isModified = false;
+    setModified(false); // Undoes changes which stem from ECO parsing, is this expected or unexpected?
     return ok;
 }
-
-bool MemoryDatabase::clear()
-{
-    for(int i = 0; i < m_games.count(); ++i)
-    {
-        delete m_games[i];
-    }
-    m_games.clear();
-    m_isModified = true;
-    return true;
-}
