@@ -176,6 +176,10 @@ void Output::readTemplateFile(const QString& path)
                 {
                     setMarkupTag(MarkupVariationInline, tags[0], tags[1]);
                 }
+                else if(name == "MarkupVariationResume")
+                {
+                    setMarkupTag(MarkupVariationResume, tags[0], tags[1]);
+                }
                 else if(name == "MarkupVariationIndent")
                 {
                     setMarkupTag(MarkupVariationIndent, tags[0], tags[1]);
@@ -258,7 +262,7 @@ void Output::readTemplateFile(const QString& path)
                 }
                 else
                 {
-                    qWarning("Unkown Markup Tag found in file %s line %d. Ignoring : %s", path.toLatin1().constData(), i,  line.toLatin1().constData());
+                    qWarning("Unknown Markup Tag found in file %s line %d. Ignoring : %s", path.toLatin1().constData(), i,  line.toLatin1().constData());
                 }
             }
             break;
@@ -280,7 +284,7 @@ QMap<Output::OutputType, QString>& Output::getFormats()
     return m_outputMap;
 }
 
-QString Output::writeDiagram(int n)
+QString Output::writeDiagram(int n) const
 {
     QString imageString;
     if((m_outputType == NotationWidget) && (AppSettings->getValue("/GameText/ShowDiagrams").toBool()))
@@ -289,7 +293,7 @@ QString Output::writeDiagram(int n)
         boardView.setMinimumSize(n, n);
         boardView.setEnabled(false);
         boardView.configure();
-        Game g = *m_game;
+        Game g = m_game;
         g.forward(1);
         boardView.setBoard(g.board());
         boardView.resize(n, n);
@@ -326,19 +330,19 @@ QString Output::writeMove(MoveToWrite moveToWrite)
     MoveId moveId;
     if(moveToWrite == NextMove)
     {
-        moveId = m_game->nextMove();
+        moveId = m_game.nextMove();
     }
     else
     {
-        moveId = m_game->currentMove();
+        moveId = m_game.currentMove();
     }
     mvno = QString::number(moveId);
-    if(m_game->nags(moveId).count() > 0)
+    if(m_game.nags(moveId).count() > 0)
     {
         if(m_options.getOptionAsBool("SymbolicNag"))
         {
-            nagString += m_game->nags(moveId).toString(m_outputType == Html ? NagSet::HTML : NagSet::Simple);
-            if((m_outputType == Html || m_outputType == NotationWidget) && (m_game->nags(moveId).contains(NagDiagram)))
+            nagString += m_game.nags(moveId).toString(m_outputType == Html ? NagSet::HTML : NagSet::Simple);
+            if((m_outputType == Html || m_outputType == NotationWidget) && (m_game.nags(moveId).contains(NagDiagram)))
             {
                 int n = m_options.getOptionAsInt("DiagramSize");
                 if(n)
@@ -349,21 +353,21 @@ QString Output::writeMove(MoveToWrite moveToWrite)
         }
         else
         {
-            nagString += m_game->nags(moveId).toString(NagSet::PGN);
+            nagString += m_game.nags(moveId).toString(NagSet::PGN);
         }
 
     }
     // Read comments
-    if(m_game->canHaveStartAnnotation(moveId))
-        precommentString = (m_outputType == Pgn) ? m_game->annotation(moveId, Game::BeforeMove) :
-                           m_game->textAnnotation(moveId, Game::BeforeMove);
+    if(m_game.canHaveStartAnnotation(moveId))
+        precommentString = (m_outputType == Pgn) ? m_game.annotation(moveId, Game::BeforeMove) :
+                           m_game.textAnnotation(moveId, Game::BeforeMove);
 
-    QString commentString = (m_outputType == Pgn) ? m_game->annotation(moveId) : m_game->textAnnotation(moveId);
+    QString commentString = (m_outputType == Pgn) ? m_game.annotation(moveId) : m_game.textAnnotation(moveId);
 
     // Write precomment if any
     text += writeComment(precommentString, mvno, Precomment);
 
-    Color c = m_game->board().toMove();
+    Color c = m_game.board().toMove();
 
     if((m_options.getOptionAsBool("ColumnStyle")) &&
             (m_currentVariationLevel == 0) &&
@@ -385,11 +389,11 @@ QString Output::writeMove(MoveToWrite moveToWrite)
     }
     if(c == White)
     {
-        text += QString::number(m_game->moveNumber(moveId)) + ". ";
+        text += QString::number(m_game.moveNumber(moveId)) + ". ";
     }
     else if(m_dirtyBlack)
     {
-        text += QString::number(m_game->moveNumber(moveId)) + "... ";
+        text += QString::number(m_game.moveNumber(moveId)) + "... ";
         if((m_options.getOptionAsBool("ColumnStyle")) &&
                 (m_currentVariationLevel == 0))
         {
@@ -427,11 +431,11 @@ QString Output::writeMove(MoveToWrite moveToWrite)
     Game::MoveStringFlags flags = (m_outputType == NotationWidget) ? Game::TranslatePiece : Game::MoveOnly;
     if(moveToWrite == NextMove)
     {
-        san = m_game->moveToSan(flags);
+        san = m_game.moveToSan(flags);
     }
     else
     {
-        san = m_game->moveToSan((Game::MoveStringFlags)(flags | Game::MoveOnly), Game::PreviousMove);
+        san = m_game.moveToSan((Game::MoveStringFlags)(flags | Game::MoveOnly), Game::PreviousMove);
     }
     QString mate = m_startTagMap[MarkupMate] + "#" + m_endTagMap[MarkupMate];
     san.replace("#", mate);
@@ -457,12 +461,12 @@ QString Output::writeMove(MoveToWrite moveToWrite)
             (c == White))
     {
         text += m_endTagMap[MarkupColumnStyleMove];
-        m_game->forward();
-        if(m_game->atGameEnd())
+        m_game.forward();
+        if(m_game.atGameEnd())
         {
             text += m_endTagMap[MarkupColumnStyleRow];
         }
-        m_game->backward();
+        m_game.backward();
     }
 
     if((m_options.getOptionAsBool("ColumnStyle")) &&
@@ -479,77 +483,112 @@ QString Output::writeMove(MoveToWrite moveToWrite)
     }
 
     text += writeComment(commentString, mvno, Comment);
-    text += " ";
+    if (m_game.hasNextMove())
+    {
+        text += " ";
+    }
     return text;
 }
 
-QString Output::writeVariation(MoveId upToNode)
+QString Output::writeMainLine(MoveId upToNode)
 {
     QString text;
-    while(!m_game->atLineEnd())
+    while(!m_game.atLineEnd())
     {
-        if(m_game->currentMove() == upToNode)
+        // Training mode: abort main line with current node
+        if(m_game.currentMove() == upToNode)
         {
-            text += "*** ";
+            if(m_options.getOptionAsBool("ColumnStyle"))
+            {
+                text += m_endTagMap[MarkupColumnStyleMainline];
+            }
+            text += "***";
             break;
         }
-        // *** Writes move in the current variation
+        // *** Write moves in the main line
         text += writeMove();
-        if(m_game->variationCount())
+        bool hasNext = m_game.hasNextMove();
+        if(m_game.variationCount())
         {
-            QList <int> variations = m_game->variations();
+            QList<MoveId> variations = m_game.variations();
             if(variations.size())
             {
-                bool inMainline = m_game->isMainline();
-                if(m_options.getOptionAsBool("ColumnStyle") && inMainline)
+                if(m_options.getOptionAsBool("ColumnStyle"))
                 {
                     text += m_endTagMap[MarkupColumnStyleMainline];
                 }
                 for(int i = 0; i < variations.size(); ++i)
                 {
-                    m_currentVariationLevel++;
-                    if(m_currentVariationLevel <= m_options.getOptionAsInt("VariationIndentLevel"))
-                    {
-                        text += m_startTagMap[MarkupVariationIndent];
-                    }
-                    else
-                    {
-                        text += m_startTagMap[MarkupVariationInline];
-                    }
-
-                    m_dirtyBlack = true;
-
                     // *** Enter variation i, and write the rest of the moves
-                    m_game->moveToId(variations[i]);
-                    text += writeMove(PreviousMove);
-                    text += writeVariation(upToNode);
-
-                    // *** End the variation
-                    //			m_output.replace ( QRegExp ("\\s+$"), "" ); // We don't want any spaces before the )
-                    if(m_currentVariationLevel <= m_options.getOptionAsInt("VariationIndentLevel"))
-                    {
-                        text += m_endTagMap[MarkupVariationIndent];
-                    }
-                    else
-                    {
-                        text += m_endTagMap[MarkupVariationInline];
-                    }
-                    m_currentVariationLevel--;
+                    m_game.moveToId(variations[i]);
+                    text += writeVariation();
                 }
-                if(m_options.getOptionAsBool("ColumnStyle") && inMainline)
+                if(hasNext && m_options.getOptionAsBool("ColumnStyle"))
                 {
                     text += m_startTagMap[MarkupColumnStyleMainline];
                 }
             }
             m_dirtyBlack = true;
-            m_game->moveToId(m_game->parentMove());
+            m_game.moveToId(m_game.parentMove());
         }
-        m_game->forward();
+        m_game.forward();
     }
+
     return text;
 }
 
-QString Output::writeTag(const QString& tagName, const QString& tagValue)
+QString Output::writeVariation()
+{
+    QString text;
+    m_currentVariationLevel++;
+    bool indent = (m_currentVariationLevel < m_options.getOptionAsInt("VariationIndentLevel"));
+    bool willStartIndent = (m_currentVariationLevel+1 == m_options.getOptionAsInt("VariationIndentLevel"));
+
+    if (indent) text += m_startTagMap[MarkupVariationResume];
+    text += m_startTagMap[indent ? MarkupVariationIndent : MarkupVariationInline];
+    m_dirtyBlack = true;
+
+    text += writeMove(PreviousMove);
+
+    bool mustAddStart = false;
+
+    while(!m_game.atLineEnd())
+    {
+        if (mustAddStart)
+        {
+            mustAddStart = false;
+            if (indent) text += m_startTagMap[MarkupVariationResume];
+        }
+        // *** Writes move in the current variation
+        text += writeMove();
+        if(m_game.variationCount())
+        {
+            if (indent && !willStartIndent) text += m_endTagMap[MarkupVariationResume];
+            QList<MoveId> variations = m_game.variations();
+            if(variations.size())
+            {
+                for(int i = 0; i < variations.size(); ++i)
+                {
+                    // *** Enter variation i, and write the rest of the moves
+                    m_game.moveToId(variations[i]);
+                    text += writeVariation();
+                }
+                mustAddStart = true;
+            }
+            m_dirtyBlack = true;
+            m_game.moveToId(m_game.parentMove());
+        }
+        m_game.forward();
+    }
+
+    text += m_endTagMap[indent ? MarkupVariationIndent : MarkupVariationInline];
+    if (indent) text += m_endTagMap[MarkupVariationResume];
+
+    m_currentVariationLevel--;
+    return text;
+}
+
+QString Output::writeTag(const QString& tagName, const QString& tagValue) const
 {
     QString text = m_startTagMap[MarkupHeaderLine] +
                    m_startTagMap[MarkupHeaderTagName] +
@@ -573,10 +612,6 @@ QString Output::writeComment(const QString& comment, const QString& mvno, Commen
     MarkupType markupIndent = type == Comment ? MarkupAnnotationIndent : MarkupPreAnnotationIndent;
     MarkupType markupInline = type == Comment ? MarkupAnnotationInline : MarkupPreAnnotationInline;
 
-    if(type == Comment)
-    {
-        text += " ";
-    }
     if(m_options.getOptionAsBool("ColumnStyle") && (m_currentVariationLevel == 0))
     {
         text += m_endTagMap[MarkupColumnStyleMainline];
@@ -617,7 +652,7 @@ QString Output::writeComment(const QString& comment, const QString& mvno, Commen
     return text;
 }
 
-QString Output::writeGameComment(QString comment)
+QString Output::writeGameComment(QString comment) const
 {
     QString text;
     comment = comment.trimmed();
@@ -652,10 +687,10 @@ QString Output::writeGameComment(QString comment)
     return text;
 }
 
-QString Output::writeAllTags()
+QString Output::writeAllTags() const
 {
     QString text;
-    QMap<QString, QString> tags = m_game->tags();
+    QMap<QString, QString> tags = m_game.tags();
     // write standard tags
     for(int i = 0; i < 7; ++i)
     {
@@ -677,10 +712,10 @@ QString Output::writeAllTags()
     return text;
 }
 
-QString Output::writeBasicTagsHTML()
+QString Output::writeBasicTagsHTML() const
 {
     QString text;
-    QMap<QString, QString> tags = m_game->tags();
+    QMap<QString, QString> tags = m_game.tags();
 
     QString eco = tags[TagNameECO].left(3);
     if(eco == "?")
@@ -715,20 +750,25 @@ QString Output::writeBasicTagsHTML()
 QString Output::output(Game* game, bool upToCurrentMove)
 {
     QString text = m_header;
-    text += outputGame(game, upToCurrentMove);
-    text += m_footer;
     postProcessOutput(text);
+
+    text += outputTags(game);
+
+    QString gameText = outputGame(game, upToCurrentMove);
+    postProcessOutput(gameText);
+    text += gameText;
+
+    QString footer = m_footer;
+    postProcessOutput(footer);
+    text += footer;
+
     return text;
 }
 
-QString Output::outputGame(Game* game, bool upToCurrentMove)
+QString Output::outputTags(Game* game)
 {
     QString text;
-    m_game = game;
-    int id = m_game->currentMove();
-    int mainId = upToCurrentMove ? m_game->mainLineMove() : NO_MOVE;
-    m_currentVariationLevel = 0;
-
+    m_game = *game;
     if(m_options.getOptionAsBool("ShowHeader"))
     {
         text += m_startTagMap[MarkupHeaderBlock];
@@ -742,9 +782,19 @@ QString Output::outputGame(Game* game, bool upToCurrentMove)
         }
         text += m_endTagMap[MarkupHeaderBlock];
     }
+    return text;
+}
 
-    m_game->moveToStart();
-    m_dirtyBlack = m_game->board().toMove() == Black;
+QString Output::outputGame(const Game* g, bool upToCurrentMove)
+{
+    QString text;
+    m_game = *g;
+    int id = m_game.currentMove();
+    int mainId = upToCurrentMove ? m_game.mainLineMove() : NO_MOVE;
+    m_currentVariationLevel = 0;
+
+    m_game.moveToStart();
+    m_dirtyBlack = m_game.board().toMove() == Black;
     text += m_startTagMap[MarkupNotationBlock];
     text += m_startTagMap[MarkupMainLine];
     if(m_options.getOptionAsBool("ColumnStyle"))
@@ -752,18 +802,18 @@ QString Output::outputGame(Game* game, bool upToCurrentMove)
         text += m_startTagMap[MarkupColumnStyleMainline];
     }
 
-    text += writeGameComment(game->gameComment());
+    text += writeGameComment(m_game.gameComment());
 
-    text += writeVariation(mainId);
+    text += writeMainLine(mainId);
     if(m_options.getOptionAsBool("ColumnStyle"))
     {
         text += m_endTagMap[MarkupColumnStyleMainline];
     }
     text += m_endTagMap[MarkupMainLine];
     text += m_endTagMap[MarkupNotationBlock];
-    text += m_startTagMap[MarkupResult] + game->tag("Result") + m_endTagMap[MarkupResult];
+    text += m_startTagMap[MarkupResult] + m_game.tag("Result") + m_endTagMap[MarkupResult];
 
-    m_game->moveToId(id);
+    m_game.moveToId(id);
 
     return text;
 }
@@ -778,7 +828,7 @@ void Output::postProcessOutput(QString& text) const
     }
 
     // Chop it up, if TextWidth option is not equal to 0
-    int start = text.length();
+    int start = 0;
     int textWidth = m_options.getOptionAsInt("TextWidth");
     if(textWidth)
     {
@@ -808,6 +858,9 @@ void Output::output(QTextStream& out, Filter& filter)
     {
         if(filter.database()->loadGame(filter.indexToGame(i), game))
         {
+            QString tagText = outputTags(&game);
+            out << tagText;
+
             QString outText = outputGame(&game, false);
             postProcessOutput(outText);
             out << outText;
@@ -842,10 +895,13 @@ void Output::output(QTextStream& out, Database& database)
 
     int percentDone = 0;
     Game game;
-    for(int i = 0; i < database.count(); ++i)
+    for(int i = 0; i < (int)database.count(); ++i)
     {
         if(database.loadGame(i, game))
         {
+            QString tagText = outputTags(&game);
+            out << tagText;
+
             QString outText = outputGame(&game, false);
             postProcessOutput(outText);
             out << outText;
