@@ -32,8 +32,8 @@ bool Settings::layout(QWidget* w)
     bool valid = list(w->objectName(), values, 5);
     if(valid)    // Enough values
     {
-        w->resize(QSize(values[2], values[3]));
-        w->move(QPoint(values[0], values[1]));
+        w->resize(values[2], values[3]);
+        w->move(values[0], values[1]);
         if(qobject_cast<QMainWindow*>(w))
         {
             QByteArray docks = value("Docks", QByteArray()).toByteArray();
@@ -42,9 +42,12 @@ bool Settings::layout(QWidget* w)
                 qobject_cast<QMainWindow*>(w)->restoreState(docks, 0);
             }
         }
-        else if(values[4])    // restore non-main windows
+        else
         {
-            w->show();
+            if(values[4])
+            {
+                w->show();
+            }
         }
     }
     endGroup();
@@ -84,6 +87,39 @@ QString Settings::dataPath()
     return m_dataPath;
 }
 
+QString Settings::programDataPath()
+{
+    QString programDataPath = QCoreApplication::applicationDirPath();
+    programDataPath.append("/data");
+    return programDataPath;
+}
+
+QString Settings::uciPath()
+{
+#ifdef Q_OS_WIN
+    QString path(AppSettings->programDataPath());
+    return (path + "/engines/uci");
+#endif
+#ifdef Q_OS_MAC
+    QString path(AppSettings->programDataPath());
+    return (path + "/engines-mac/uci");
+#endif
+    return "";
+}
+
+QString Settings::winboardPath()
+{
+#ifdef Q_OS_WIN
+    QString path(AppSettings->programDataPath());
+    return (path + "/engines/winboard");
+#endif
+#ifdef Q_OS_MAC
+    QString path(AppSettings->programDataPath());
+    return (path + "/engines-mac/winboard");
+#endif
+    return "";
+}
+
 QString Settings::commonDataPath()
 {
 #if QT_VERSION < 0x050000
@@ -101,7 +137,9 @@ void Settings::setList(const QString& key, QList<int> list)
     QList<QVariant> varlist;
     int i;
     foreach(i, list)
-    varlist.append(QVariant(i));
+    {
+        varlist.append(QVariant(i));
+    }
     setValue(key, varlist);
 }
 
@@ -114,7 +152,9 @@ bool Settings::list(const QString &key, QList<int>& list, int items)
     }
     QVariant v;
     foreach(v, varlist)
-    list.append(v.toInt());
+    {
+        list.append(v.toInt());
+    }
     return true;
 }
 
@@ -150,6 +190,8 @@ void Settings::getMap(const QString& key, OptionValueList& map)
 QMap<QString, QVariant> Settings::initDefaultValues() const
 {
     QMap<QString, QVariant> map;
+    map.insert("ColumnCount", -1);
+
     map.insert("/General/EditLimit", 10);
     map.insert("/General/automaticECO", true);
     map.insert("/General/useIndexFile", true);
@@ -175,16 +217,22 @@ QMap<QString, QVariant> Settings::initDefaultValues() const
     map.insert("/GameText/ShowHeader", false);
     map.insert("/GameText/ShowDiagrams", true);
     map.insert("/GameText/DiagramSize", 200);
-
+    map.insert("/GameText/FontBrowserText", "'Arial Unicode MS',Menlo");
+    map.insert("/GameText/FontBrowserMove", "'Arial Unicode MS',Menlo");
     map.insert("/GameText/PieceString", " KQRBN");
+
     map.insert("/MainWindow/GameToolBar", false);
     map.insert("/MainWindow/VerticalTabs", false);
     map.insert("/MainWindow/StayOnTop", false);
     map.insert("/MainWindow/FilterFollowsGame", false);
+    map.insert("/MainWindow/ShowMenuIcons", true);
+
     map.insert("/History/MaxEntries", 4);
+
     map.insert("/Board/showFrame", true);
     map.insert("/Board/showCoordinates", true);
     map.insert("/Board/showCurrentMove", true);
+    map.insert("/Board/showMoveIndicator", 0);
     map.insert("/Board/guessMove", true);
     map.insert("/Board/nextGuess", false);
     map.insert("/Board/minWheelCount", MIN_WHEEL_COUNT);
@@ -198,6 +246,8 @@ QMap<QString, QVariant> Settings::initDefaultValues() const
     map.insert("/Board/currentMoveColor", QColor(Qt::blue));
     map.insert("/Board/AutoPlayerInterval", 3000);
     map.insert("/Board/AutoSaveAndContinue", false);
+    map.insert("/Tools/Path1", "");
+    map.insert("/Tools/CommandLine1", "");
     return map;
 }
 
@@ -216,8 +266,7 @@ QVariant Settings::getValue(const QString &key) const
             return value(key, defaultValues.value(groupKey));
         }
     }
-    Q_ASSERT(false);
-    return QVariant();
+    return value(key);
 }
 
 QString Settings::getThemaPath() const
