@@ -28,6 +28,8 @@ BoardSetupDialog::BoardSetupDialog(QWidget* parent, Qt::WindowFlags f) : QDialog
 {
     setObjectName("BoardSetupDialog");
     ui.setupUi(this);
+    restoreLayout();
+
     ui.boardView->configure();
     ui.boardView->setFlags(BoardView::IgnoreSideToMove | BoardView::SuppressGuessMove | BoardView::AllowCopyPiece);
     ui.boardView->showMoveIndicator(false);
@@ -38,7 +40,6 @@ BoardSetupDialog::BoardSetupDialog(QWidget* parent, Qt::WindowFlags f) : QDialog
     {
         BoardSetupToolButton* button = new BoardSetupToolButton(this);
         button->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
-        //button->setAlignment(Qt::AlignJustify|Qt::AlignVCenter);
         button->setMinimumSize(QSize(10, 10));
         button->m_piece = (Piece)piece;
         if(piece == Empty)
@@ -58,7 +59,7 @@ BoardSetupDialog::BoardSetupDialog(QWidget* parent, Qt::WindowFlags f) : QDialog
 
     emit signalClearBackground(Empty);
 
-    connect(ui.okButton, SIGNAL(clicked()), SLOT(slotAccept()));
+    connect(ui.okButton, SIGNAL(clicked()), SLOT(accept()));
     connect(ui.cancelButton, SIGNAL(clicked()), SLOT(reject()));
     connect(ui.clearButton, SIGNAL(clicked()), SLOT(slotClear()));
     connect(ui.resetButton, SIGNAL(clicked()), SLOT(slotReset()));
@@ -89,6 +90,11 @@ BoardSetupDialog::BoardSetupDialog(QWidget* parent, Qt::WindowFlags f) : QDialog
     ui.tabWidget->setCurrentIndex(0);
 }
 
+void BoardSetupDialog::restoreLayout()
+{
+    AppSettings->layout(this);
+}
+
 BoardSetupDialog::~BoardSetupDialog()
 {}
 
@@ -103,7 +109,7 @@ void BoardSetupDialog::mirrorVertical()
     QString fen = board().toFen();
     QString fenRows = fen.left(fen.indexOf(" "));
     QStringList rows = fenRows.split("/");
-    std::reverse(rows.begin(), rows.end());
+    QAlgorithmsPrivate::qReverse(rows.begin(), rows.end());
     QString newFen = rows.join("/");
     Board b(newFen);
     b.setMoveNumber(board().moveNumber());
@@ -118,7 +124,7 @@ void BoardSetupDialog::mirrorHorizontal()
     QStringList rows = fenRows.split("/");
     for(QStringList::Iterator iter = rows.begin(); iter != rows.end(); ++iter)
     {
-        std::reverse((*iter).begin(), (*iter).end());
+        QAlgorithmsPrivate::qReverse((*iter).begin(), (*iter).end());
     }
     QString newFen = rows.join("/");
     Board b(newFen);
@@ -204,17 +210,19 @@ void BoardSetupDialog::slotReset()
     setBoard(b);
 }
 
-void BoardSetupDialog::slotAccept()
+void BoardSetupDialog::accept()
 {
     // Need to make sure the board is updated with move number set by user
     Board b(ui.boardView->board());
     b.setMoveNumber(ui.moveSpin->value());
     ui.boardView->setBoard(b);
 
+    AppSettings->setLayout(this);
+
     QString reason = boardStatusMessage();
     if(reason.isEmpty())
     {
-        accept();
+        QDialog::accept();
     }
     else
     {
@@ -222,6 +230,12 @@ void BoardSetupDialog::slotAccept()
     }
 }
 
+void BoardSetupDialog::reject()
+{
+    AppSettings->setLayout(this);
+    QDialog::reject();
+}
+
 void BoardSetupDialog::slotClear()
 {
     Board b;
