@@ -25,6 +25,7 @@
 using namespace Qt;
 
 const int CoordinateSize = 16;
+const int MoveIndicatorSize = 12;
 
 BoardView::BoardView(QWidget* parent, int flags) : QWidget(parent),
     m_flipped(false), m_showFrame(false), m_showCurrentMove(true),
@@ -35,7 +36,7 @@ BoardView::BoardView(QWidget* parent, int flags) : QWidget(parent),
     m_atLineEnd(true),
     m_flags(flags),
     m_coordinates(false), m_dragged(Empty), m_clickUsed(false), m_wheelCurrentDelta(0),
-    m_minDeltaWheel(0), m_moveListCurrent(0), m_showMoveIndicator(true), m_DbIndex(0)
+    m_minDeltaWheel(0), m_moveListCurrent(0), m_showMoveIndicator(true), m_vAlignTop(false), m_DbIndex(0)
 {
     QSizePolicy policy(QSizePolicy::Preferred, QSizePolicy::Preferred);
     policy.setHeightForWidth(true);
@@ -64,7 +65,6 @@ void BoardView::setFlags(int flags)
 void BoardView::setBoard(const Board& value, int from, int to, bool atLineEnd)
 {
     m_clickUsed = true;
-    Board oldboard = m_board;
     m_board = value;
     m_currentFrom = from;
     m_currentTo = to;
@@ -94,6 +94,7 @@ void BoardView::showMoveIndicator(bool visible)
 void BoardView::drawSquares(QPaintEvent* event)
 {
     QPainter p(this);
+    p.translate(m_translate);
     for(Square square = 0; square < 64; square++)
     {
         QRect rect = squareRect(square);
@@ -150,22 +151,18 @@ void BoardView::drawDraggedPieces(QPaintEvent* /*event*/)
 
 void BoardView::drawMoveIndicator(QPaintEvent* event)
 {
-    if(m_showMoveIndicator)
+    if(m_showMoveIndicator && m_showMoveIndicatorMode!=Never)
     {
         QPainter p(this);
         // Draw side to move indicator
         int coord =  m_coordinates ? CoordinateSize : 0;
         bool white = m_board.toMove() == White;
-        int square = width() - coord - 8 * m_theme.size().width() - 4;
-        int maxsquare = m_theme.size().width() / 2;
-        if(square > maxsquare)
-        {
-            square = maxsquare;
-        }
-        if(square > 8)
+        int square = moveIndicatorWidth();
+        if(square)
         {
             int posy = (white == m_flipped) ? 1 : 8 * m_theme.size().width() - square;
-            QRect rect(8 * m_theme.size().width() + 2 + coord, posy, square, square);
+            QRect rect(QPoint(8 * m_theme.size().width() + 2 + coord, posy) + m_translate,
+                       QSize(square, square));
             if(!event->region().intersects(rect))
             {
                 return;
@@ -182,6 +179,7 @@ void BoardView::drawMoveIndicator(QPaintEvent* event)
 void BoardView::drawPieces(QPaintEvent* event)
 {
     QPainter p(this);
+    p.translate(m_translate);
 
     for(Square square = 0; square < 64; square++)
     {
@@ -245,10 +243,19 @@ void BoardView::resizeBoard(QSize sz)
 {
     // subtract move indicator from width
     int coord = m_coordinates ? CoordinateSize : 0;
-    int xsize = (sz.width() - 1 - coord) / 8;
+    int moveIndicator = (m_showMoveIndicatorMode==Always) ? MoveIndicatorSize:0;
+    int xsize = (sz.width() - 1 - coord - moveIndicator) / 8;
     int ysize = (sz.height() - 1 - coord) / 8;
     int size = std::min(xsize, ysize);
     m_theme.setSize(QSize(size, size));
+
+    // calculate the translation vector to center the widget inside its parent
+    int square = moveIndicatorWidth();
+
+    QSize widgetSize(size * 8 + 1 + coord + square, size * 8 + 1 + coord);
+    QPoint widgetCenter(widgetSize.width() / 2, widgetSize.height() / 2);
+    m_translate = geometry().center() - widgetCenter - geometry().topLeft();
+    if (m_vAlignTop) m_translate.setY(0);
 }
 
 void BoardView::resizeEvent(QResizeEvent* e)
@@ -256,9 +263,31 @@ void BoardView::resizeEvent(QResizeEvent* e)
     resizeBoard(e->size());
 }
 
+int BoardView::moveIndicatorWidth() const
+{
+    if(m_showMoveIndicator && m_showMoveIndicatorMode!=Never)
+    {
+        int coord = m_coordinates ? CoordinateSize : 0;
+        int square = width() - coord - 8 * m_theme.size().width() - 4;
+        int maxsquare = m_theme.size().width() / 2;
+        if (square > maxsquare)
+        {
+            square = maxsquare;
+        }
+        if (square <= 8)
+        {
+            square = 0;
+        }
+        return square;
+    }
+    return 0;
+}
+
+
 Square BoardView::squareAt(const QPoint& p) const
 {
-    int x = p.x(), y = p.y();
+    int x = p.x() - m_translate.x();
+    int y = p.y() - m_translate.y();
     int width = m_theme.size().width();
     int height = m_theme.size().height();
     x -= m_coordinates ? CoordinateSize : 0;
@@ -274,6 +303,7 @@ Square BoardView::squareAt(const QPoint& p) const
 void BoardView::mousePressEvent(QMouseEvent* event)
 {
     m_dragStart = event->pos();
+    emit moveStarted();
 }
 
 bool BoardView::showGuess(Square s)
@@ -285,7 +315,7 @@ bool BoardView::showGuess(Square s)
         m_hoverSquare = s;
         removeGuess();
         m_moveListCurrent = 0;
-        m_moveList.Clear();
+        m_moveList.clear();
 #ifdef USE_ECO_GUESS
         if(m_board.ecoMove(s, &m_hifrom, &m_hito))
         {
@@ -330,13 +360,13 @@ void BoardView::nextGuess(Square s)
 {
     if(!showGuess(s))
     {
-        if(m_moveList.Size() && m_moveListCurrent < m_moveList.Size())
+        if(m_moveList.size() && (int) m_moveListCurrent < m_moveList.size())
         {
             Guess::simpleMoveT * sold = m_moveList.Get(m_moveListCurrent);
             update(squareRect(sold->from));
             update(squareRect(sold->to));
 
-            if(m_moveListCurrent < m_moveList.Size() - 1)
+            if((int)m_moveListCurrent < m_moveList.size() - 1)
             {
                 ++m_moveListCurrent;
             }
@@ -449,6 +479,7 @@ void BoardView::mouseReleaseEvent(QMouseEvent* event)
             emit invalidMove(from);
         }
         m_dragged = Empty;
+        emit moveFinished();
         return;
     }
     else
@@ -459,6 +490,7 @@ void BoardView::mouseReleaseEvent(QMouseEvent* event)
             {
                 emit clicked(s, button, mapToGlobal(event->pos()), InvalidSquare);
             }
+            emit moveFinished();
             return;
         }
     }
@@ -524,6 +556,7 @@ void BoardView::mouseReleaseEvent(QMouseEvent* event)
             }
         }
     }
+    emit moveFinished();
 }
 
 void BoardView::wheelEvent(QWheelEvent* e)
@@ -562,6 +595,7 @@ void BoardView::configure()
     m_showCurrentMove = AppSettings->getValue("showCurrentMove").toBool();
     m_guessMove = AppSettings->getValue("guessMove").toBool();
     m_minDeltaWheel = AppSettings->getValue("minWheelCount").toInt();
+    m_showMoveIndicatorMode = AppSettings->getValue("showMoveIndicator").toInt();
     AppSettings->endGroup();
     m_theme.configure();
     m_theme.setEnabled(isEnabled());
@@ -606,24 +640,25 @@ QRect BoardView::squareRect(Square square)
     int x = isFlipped() ? 7 - square % 8 : square % 8;
     int y = isFlipped() ? square / 8 : 7 - square / 8;
     return QRect(QPoint(x * m_theme.size().width() + coord,
-                        y * m_theme.size().height()),  m_theme.size());
+                        y * m_theme.size().height()) + m_translate,
+                 m_theme.size());
 }
 
 QRect BoardView::coordinateRectVertical(Square square)
 {
     Q_ASSERT(m_coordinates);
     int x = isFlipped() ? square % 8 : 7 - square % 8;
-    return QRect(0, x * m_theme.size().height() + (m_theme.size().height() - CoordinateSize) / 2,
-                 CoordinateSize, CoordinateSize);
+    return QRect(QPoint(0, x * m_theme.size().height() + (m_theme.size().height() - CoordinateSize) / 2) + m_translate,
+                 QSize(CoordinateSize, CoordinateSize));
 }
 
 QRect BoardView::coordinateRectHorizontal(Square square)
 {
     Q_ASSERT(m_coordinates);
     int y = isFlipped() ? 7 - square % 8 : square % 8;
-    return QRect(CoordinateSize + (y * (m_theme.size().width())) + (m_theme.size().width() - CoordinateSize) / 2,
-                 8 * m_theme.size().height(),
-                 CoordinateSize, CoordinateSize);
+    return QRect(QPoint(CoordinateSize + (y * (m_theme.size().width())) + (m_theme.size().width() - CoordinateSize) / 2,
+                        8 * m_theme.size().height()) + m_translate,
+                 QSize(CoordinateSize, CoordinateSize));
 }
 
 bool BoardView::canDrag(Square s) const
@@ -720,6 +755,7 @@ void BoardView::drawArrowAnnotations(QPaintEvent* event)
 void BoardView::drawSquareAnnotation(QPaintEvent* event, QString annotation)
 {
     QPainter p(this);
+    p.translate(m_translate);
     QString trimmed = annotation.simplified();
     QChar colorChar = trimmed[0];
     QChar fileChar = trimmed[1];
@@ -760,7 +796,7 @@ void BoardView::drawSquareAnnotation(QPaintEvent* event, QString annotation)
     p.setPen(pen);
     p.setBrush(brush);
 
-    p.setOpacity(.20);
+    p.setOpacity(.35);
 
     int coord =  m_coordinates ? CoordinateSize : 0;
     p.drawRect(coord + pos.x(), pos.y(), m_theme.size().width(), m_theme.size().height());
@@ -772,6 +808,7 @@ void BoardView::drawSquareAnnotation(QPaintEvent* event, QString annotation)
 void BoardView::drawArrowAnnotation(QPaintEvent* event, QString annotation)
 {
     QPainter p(this);
+    p.translate(m_translate);
 
     static const QString letters = "abcdefgh";
     static const QString numbers = "12345678";
@@ -866,26 +903,36 @@ void BoardView::drawArrowAnnotation(QPaintEvent* event, QString annotation)
     arrowPts[2].setY(py2);
 
     QPoint pos3(px1, py1);
-    QPoint pos4(px2, py2);
+    QPoint pos4(px2- ((adjust * dX) / arrowLength), py2- ((adjust * dY) / arrowLength));
 
     p.save();
+    p.setRenderHint(QPainter::SmoothPixmapTransform);
+
+    color.setAlpha(176);
     QPen pen(color);
-    int penWidth = std::max(1, (int)(headWidth / 8));
+    int penWidth = std::max(2, (int)(headWidth / 3));
     pen.setWidth(penWidth);
     p.setPen(pen);
-
     p.drawLine(pos3, pos4);
 
-    // For now only draw part of the arrowhead
-    //p.drawLine(arrowPts[2],arrowPts[1]);
-    //p.drawLine(arrowPts[2],arrowPts[3]);
-
     QBrush brush(color);
     p.setBrush(brush);
-
+    pen.setWidth(2);
+    p.setPen(pen);
     p.drawPolygon(arrowPts, 4);
+
     p.restore();
 }
+bool BoardView::vAlignTop() const
+{
+    return m_vAlignTop;
+}
+
+void BoardView::setVAlignTop(bool vAlignTop)
+{
+    m_vAlignTop = vAlignTop;
+}
+
 
 void BoardView::setEnabled(bool enabled)
 {
