@@ -35,6 +35,7 @@ class BoardView : public QWidget
     Q_OBJECT
 public:
     enum {WheelUp = Qt::LeftButton, WheelDown = Qt::RightButton};
+    enum {Automatic = 0, Always = 1, Never = 2};
     enum {IgnoreSideToMove = 1, SuppressGuessMove = 2, AllowCopyPiece = 4};
     /** Create board widget. */
     BoardView(QWidget* parent = 0, int flags = 0);
@@ -62,6 +63,11 @@ public:
     void setDbIndex(int);
     /** Get a reference to the database to which the current view is associated */
     int dbIndex() const;
+    /** Get the move indicator width */
+    int moveIndicatorWidth() const;
+
+    bool vAlignTop() const;
+    void setVAlignTop(bool vAlignTop);
 
 public slots:
 
@@ -89,6 +95,10 @@ signals:
     void wheelScrolled(int dir);
     /** Indicate that a piece was dropped to the board */
     void pieceDropped(Square to, Piece p);
+
+    void moveStarted();
+    void moveFinished();
+
 protected:
     /** Redraws whole board if necessary. */
     virtual void paintEvent(QPaintEvent*);
@@ -148,6 +158,7 @@ private:
 
     Board m_board;
     BoardTheme m_theme;
+    QPoint m_translate;
     bool m_flipped;
     bool m_showFrame;
     bool m_showCurrentMove;
@@ -171,6 +182,8 @@ private:
     Guess::MoveList m_moveList;
     unsigned int m_moveListCurrent;
     bool m_showMoveIndicator;
+    bool m_vAlignTop;
+    int m_showMoveIndicatorMode;
     int m_DbIndex;
 };
 
