@@ -12,6 +12,7 @@
 
 #include "common.h"
 #include "historylist.h"
+#include "kbaction.h"
 #include "move.h"
 #include "output.h"
 
@@ -41,12 +42,14 @@ class QLabel;
 class QNetworkAccessManager;
 class QNetworkReply;
 class QProgressBar;
+class QSlider;
 class QSplitter;
 class QTimer;
 class QToolBar;
 class SaveDialog;
 class TableView;
 class ToolMainWindow;
+class TranslatingSlider;
 
 /**
 @defgroup GUI GUI - User interface components
@@ -72,6 +75,8 @@ protected:
     void closeEvent(QCloseEvent* e);
     /** Filter key events. */
     void keyPressEvent(QKeyEvent *e);
+    /** Filter key events. */
+    void evaluateSanNag(QKeyEvent *e);
     /** @return active database */
     Database* database();
     /** @return active database structure */
@@ -80,6 +85,8 @@ protected:
     QString databaseName(int index = -1) const;
     /** @return active database structure */
     const DatabaseInfo* databaseInfo() const;
+    /** @return database structure */
+    DatabaseInfo* databaseInfo(QString name);
     /** @return active game */
     Game& game();
     const Game& game() const;
@@ -91,15 +98,11 @@ protected:
     QString exportFileName(int& format);
     /** Load game @p index. If @p force is false, does nothing for incorrect @p index .
     With @p force set, creates empty game. */
-    void gameLoad(int index, bool force = false, bool reload = 0);
+    void gameLoad(int index);
     /** Make given number of moves in current game */
     bool gameMoveBy(int change);
     /** Update recent files menu */
     void updateMenuRecent();
-    /** Update recent files menu */
-    void updateMenuDatabases();
-    /** Save Game dialog (created when used first) */
-    SaveDialog* saveDialog();
 public slots:
     /** Open database */
     void openDatabase(QString fname);
@@ -160,7 +163,7 @@ public slots:
     {
         gameMoveBy(-1);
     }
-    /** Go to first move of the game */
+    /** Go to last move of the game */
     void slotGameMoveLast()
     {
         gameMoveBy(999);
@@ -223,7 +226,7 @@ public slots:
     /** Add variation to current position. */
     void slotGameAddVariation(const Analysis& analysis);
     /** Add variation to current position. */
-    void slotGameAddVariation(const QString& san);
+    bool slotGameAddVariation(const QString& san);
     /** Remove all comments. */
     void slotGameUncomment();
     /** Remove all variations */
@@ -234,6 +237,8 @@ public slots:
     void slotToggleAutoAnalysis();
     /** Auto Play Timeout - make next move! */
     void slotAutoPlayTimeout();
+    /** Auto Play Timeout - make next move! */
+    void slotEngineTimeout(const Analysis& analysis);
     /** Filter was changed - update status bar information */
     void slotFilterChanged();
     /** Load given game (triggered from Game List) */
@@ -269,25 +274,28 @@ public slots:
     /** Receives the signal of a search board operation started */
     void slotBoardSearchStarted();
     /** Receives the signal of a search board operation end */
-    void slotBoardSearchUpdate();
+    void slotBoardSearchUpdate(int progress);
+    void slotBoardSearchFinished();
     /** Reset current filter */
     void slotSearchReset();
     /** Training mode */
     void slotToggleTraining();
-    /** Toggle filter */
-    void slotToggleFilter();
+    /** Training mode - auto responder */
+    void slotToggleAutoRespond();
     /** Reverse current filter */
     void slotSearchReverse();
     /** Make an Update of the game list after the opening tree was updated */
     void slotTreeUpdate();
-    /** Indicate start of tree update */
-    void slotTreeUpdateStarted();
     /** Show opening tree */
     void slotSearchTree();
     /** Move @p index was selected in Opening Tree. */
     void slotSearchTreeMove(const QModelIndex& index);
     /** Made given move on the board */
     void slotBoardMove(Square from, Square to, int button);
+    /** Move started - use together with moveFinished to avoid automatic moves during user interaction */
+    void slotMoveStarted();
+    /** Move finished - use together with moveStarted to avoid automatic moves during user interaction */
+    void slotMoveFinished();
     /** Board square was clicked */
     void slotBoardClick(Square square, int button, QPoint pos, Square from);
     /** Browse current game by mouse wheel */
@@ -302,7 +310,7 @@ public slots:
     /** Change database. */
     void slotDatabaseChange();
     /** Copy games between databases. */
-    void slotDatabaseCopy(int preselect = 1);
+    void slotDatabaseCopy(int preselect = 1, int index = -1);
     /** Copy games between databases. */
     void slotDatabaseCopySingle(int n);
     /** Database was changed - change informations. */
@@ -313,6 +321,8 @@ public slots:
     void slotDatabaseDeleteFilter();
     /** Slot that updates internal info upon loading a complete db */
     void slotDataBaseLoaded(DatabaseInfo* db);
+    /** Restore game state from a undo or redo operation */
+    void slotDbRestoreState(const Game&);
     /** Fill up the current game (drag request from game list) */
     void slotGetGameData(Game& g);
     /** Copy game from other database by drag'n'drop */
@@ -325,8 +335,19 @@ public slots:
     void slotRenamePlayer(QString ts);
     /** Pass a tag changing request from rename dialog to database */
     void slotRenameRequest(QString tag, QString newValue, QString oldValue);
-
+    /** Export an image to a file */
+    void slotExportImage();
+    /** Build a polyglot from the database @p s */
+    void slotMakeBook(QString s);
+    /** Show a path in finder */
+    void slotShowInFinder(QString path);
+    /** Show a path in finder */
+    void slotBookBuildError(QString path);
+    /** Merge the clipboard into the current game */
+    void slotEditMergePGN();
 protected slots:
+    /** Update recent files menu */
+    void updateMenuDatabases();
     /** Receiver for a failed loading of a database */
     void loadError(QUrl url);
     /** Receiver for a successful loading of a database */
@@ -359,13 +380,32 @@ protected slots:
     void slotScreenShot();
     /** Make a screenshot and save it to file */
     void slotCompileECO();
+    /** Support dragging things to non-top tabs */
+    void slotAutoSwitchTab();
+    /** Edit Accelerators */
+    void slotEditActions();
+    /** Value for Move Interval is changed */
+    void slotMoveIntervalChanged(int);
+    /** Value for Move Interval Slider is changed */
+    void slotSetSliderText(int);
 
 protected:
+    void moveChanged();
     bool pasteFen(QString& errorText, QString fen);
     void copyGame(int target, int index);
     Database* getDatabaseByPath(QString path);
     DatabaseInfo* getDatabaseInfoByPath(QString path);
     void activateBoardView(int n);
+    void updateOpeningTree(const Board& b, bool atEnd);
+
+protected:
+    void dragEnterEvent(QDragEnterEvent *event);
+    void dragMoveEvent(QDragMoveEvent *event);
+    void dragLeaveEvent(QDragLeaveEvent *event);
+    void dropEvent(QDropEvent *event);
+    int m_tabDragIndex;
+    QTabBar* m_pDragTabBar;
+    QTimer* m_dragTimer;
 
 signals:
     /** Re-read configuration. */
@@ -385,6 +425,7 @@ signals:
     void signalMoveHasNextMove(bool);
     void signalMoveHasParent(bool);
     void signalGameIsEmpty(bool);
+    void signalGameAtLineStart(bool);
 
     void signalCurrentDBisReadWrite(bool);
     void signalCurrentDBcanBeClosed(bool);
@@ -392,22 +433,34 @@ signals:
 
     void signalFirstGameLoaded(bool);
     void signalLastGameLoaded(bool);
+    void signalGameLoaded();
 
     void signalVersionFound(int, int, int);
 
     void databaseModified();
 
+    void signalDatabaseOpenClose();
+
+    void enterText(QString);
+
 private slots:
     /** Cleanup after ECO file loaded. */
     void ecoLoaded(QObject*, bool);
     void slotDatabaseModified();
     void slotHttpDone(QNetworkReply *reply);
     void slotVersionFound(int major, int minor, int build);
+    void slotUpdateOpeningTreeWidget();
 
 private:
     /** Create single menu action. */
-    QAction* createAction(const QString& name, const char* slot, const QKeySequence& key = QKeySequence(),
+    QAction* createAction(QString name, const char* slot, const QKeySequence& key = QKeySequence(),
+                          QToolBar* pToolBar = 0, QString image = QString(), const QString& tip = QString(), QAction::MenuRole menuRole = QAction::NoRole, QObject *parent=0);
+    /** Create single menu action. */
+    QAction* createAction(QObject *parent, QString name, const char* slot, const QKeySequence& key = QKeySequence(),
                           QToolBar* pToolBar = 0, QString image = QString(), const QString& tip = QString(), QAction::MenuRole menuRole = QAction::NoRole);
+    /** Create single menu action. */
+    QAction* createAction(QString name, const char* slot, const QKeySequence& key,
+                          QToolBar* pToolBar, QIcon icon = QIcon(), const QString& tip = QString(), QAction::MenuRole menuRole = QAction::NoRole, QObject *parent=0);
     /** Create all the menus and actions */
     void setupActions();
     /** Confirm quitting and save modified databases. */
@@ -422,13 +475,13 @@ private:
     void restoreRecentFiles();
     /** Query User and save game if game was modified
         @return true if the next action shall be performed */
-    bool QuerySaveGame();
+    bool QuerySaveGame(DatabaseInfo* dbInfo = 0);
     /** Save game without query */
-    void saveGame();
+    void saveGame(DatabaseInfo *dbInfo);
     /** Load next game without query */
     void loadNextGame();
     /** Save Database without query or progress bar */
-    void saveDatabase();
+    void saveDatabase(DatabaseInfo *dbInfo);
     /** Save Database with query */
     bool QuerySaveDatabase();
     /** Create a new unassociated boardview */
@@ -450,19 +503,21 @@ private:
     ECOListWidget* m_ecoList;
     EventListWidget* m_eventList;
     DatabaseList* m_databaseList;
-    SaveDialog* m_saveDialog;
     /* Main gui parts */
     BoardView* m_boardView;
     QSplitter* m_boardSplitter;
     ChessBrowser* m_gameView;
     OpeningTreeWidget* m_openingTreeWidget;
-    QProgressBar* m_progressBar;
+    QPointer<QProgressBar> m_progressBar;
+    QPointer<TranslatingSlider> m_sliderSpeed;
+    QLabel* m_sliderText;
     ToolMainWindow* m_gameWindow;
     QToolBar* m_gameToolBar;
     QTabWidget* m_tabWidget;
     QList<BoardView*> m_boardViews;
     /* Status */
     QLabel* m_statusFilter;
+    QLabel* m_statusApp;
     QLabel* m_gameTitle;
     /* Menus */
     QMenu* m_menuDatabases;
@@ -477,13 +532,10 @@ private:
     QList<QAction*> m_databaseActions;
     int m_currentDatabase;
     QString m_eco;
-    QActionGroup* m_actions;
     QTime m_operationTime;
     /** Currently updated tree. May be NULL if no update in progress. */
     QString m_nagText;
     QTimer* m_autoPlayTimer;
-    QAction* m_toggleFilter;
-    bool m_bGameChange;
     int m_currentFrom;
     int m_currentTo;
     AnalysisWidget* m_mainAnalysis;
@@ -491,11 +543,14 @@ private:
     Square m_annotationSquare;
     Square m_annotationSquareFrom;
     QAction* m_training;
+    QAction* m_autoRespond;
     QAction* m_autoPlay;
     QAction* m_autoAnalysis;
     QUndoGroup m_undoGroup;
     QNetworkAccessManager* m_manager;
     DownloadManager* downloadManager;
+    bool m_machineHasToMove;
+    bool m_bInDrag;
 };
 
 
