@@ -24,6 +24,7 @@
 #include <QCheckBox>
 #include <QColorDialog>
 #include <QComboBox>
+#include <QFontDialog>
 #include <QListWidget>
 #include <QPainter>
 #include <QDir>
@@ -66,6 +67,7 @@ PreferencesDialog::PreferencesDialog(QWidget* parent, Qt::WindowFlags f) : QDial
     connect(ui.tbSymbolic, SIGNAL(clicked()), SLOT(slotChangePieceString()));
 
     connect(ui.btLoadLang, SIGNAL(clicked()), SLOT(slotLoadLanguageFile()));
+    connect(ui.btExtToolPath, SIGNAL(clicked(bool)), SLOT(slotSelectToolPath()));
     restoreSettings();
 
     // Start off with no Engine selected
@@ -87,6 +89,12 @@ PreferencesDialog::PreferencesDialog(QWidget* parent, Qt::WindowFlags f) : QDial
     {
         ui.labelLoadStatus->setText(tr("For updating translations online version checking needs to be enabled."));
     }
+    QTimer::singleShot(0, this, SLOT(restoreLayout()));
+}
+
+void PreferencesDialog::restoreLayout()
+{
+    AppSettings->layout(this);
 }
 
 PreferencesDialog::~PreferencesDialog()
@@ -95,13 +103,10 @@ PreferencesDialog::~PreferencesDialog()
 
 void PreferencesDialog::done(int r)
 {
+    AppSettings->setLayout(this);
     QDialog::done(r);
     close();
 }
-void PreferencesDialog::closeEvent(QCloseEvent*)
-{
-    AppSettings->setLayout(this);
-}
 
 void PreferencesDialog::slotSelectEngineDirectory()
 {
@@ -114,6 +119,16 @@ void PreferencesDialog::slotSelectEngineDirectory()
     }
 }
 
+void PreferencesDialog::slotSelectToolPath()
+{
+    QString fileName = QFileDialog::getOpenFileName(this, tr("Select external executable"),
+                                                    ui.extToolPath->text());
+    if(QFileInfo(fileName).exists())
+    {
+        ui.extToolPath->setText(fileName);
+    }
+}
+
 void PreferencesDialog::slotSelectDataBasePath()
 {
     QString dir = QFileDialog::getExistingDirectory(this,
@@ -263,7 +278,7 @@ void PreferencesDialog::slotShowOptionDialog()
 {
     int index = ui.engineList->currentIndex().row();
     updateEngineData(index);
-    EngineOptionDialog dlg(0, engineList, index);
+    EngineOptionDialog dlg(this, engineList, index);
     if(dlg.exec() == QDialog::Accepted)
     {
         engineList[index].m_optionValues = dlg.GetResults();
@@ -388,6 +403,7 @@ void PreferencesDialog::slotReset()
     if(MessageDialog::yesNo(tr("Clear all application settings?"), tr("Warning")))
     {
         AppSettings->clear();
+        AppSettings->setLayout(this);
         restoreSettings();
         emit reconfigure();
     }
@@ -401,9 +417,6 @@ void PreferencesDialog::slotApply()
 
 void PreferencesDialog::restoreSettings()
 {
-    // Restore size
-    AppSettings->layout(this);
-
     // Read Board settings
     AppSettings->beginGroup("/General/");
     ui.tablebaseCheck->setChecked(AppSettings->getValue("onlineTablebases").toBool());
@@ -417,10 +430,10 @@ void PreferencesDialog::restoreSettings()
     ui.boardFrameCheck->setChecked(AppSettings->getValue("showFrame").toBool());
     ui.boardShowCoordinates->setChecked(AppSettings->getValue("showCoordinates").toBool());
     ui.hilightCurrentMove->setChecked(AppSettings->getValue("showCurrentMove").toBool());
+    ui.cbShowIndicator->setCurrentIndex(AppSettings->getValue("showMoveIndicator").toInt());
     ui.guessMoveCheck->setChecked(AppSettings->getValue("guessMove").toBool());
     ui.guessNextMove->setChecked(AppSettings->getValue("nextGuess").toBool());
     ui.minWheelCount->setValue(AppSettings->getValue("minWheelCount").toInt());
-    ui.autoPlayInterval->setValue(AppSettings->getValue("AutoPlayerInterval").toInt());
     ui.cbSaveAndContinue->setChecked(AppSettings->getValue("AutoSaveAndContinue").toBool());
     QString pieceTheme = AppSettings->getValue("pieceTheme").toString();
     ui.pieceEffect->setCurrentIndex(AppSettings->getValue("pieceEffect").toInt());
@@ -479,7 +492,7 @@ void PreferencesDialog::restoreSettings()
     ui.defaultDataBasePath->setText(AppSettings->value("/General/DefaultDataPath", dataPath).toString());
     ui.spinBoxListFontSize->setValue(AppSettings->getValue("/General/ListFontSize").toInt());
     ui.verticalTabs->setChecked(AppSettings->getValue("/MainWindow/VerticalTabs").toBool());
-
+    ui.iconsVisible->setChecked(AppSettings->getValue("/MainWindow/ShowMenuIcons").toBool());
     // Read Game List settings
     AppSettings->beginGroup("GameText");
 
@@ -496,6 +509,14 @@ void PreferencesDialog::restoreSettings()
     ui.diagramSize->setValue(AppSettings->getValue("DiagramSize").toInt());
     ui.pieceString->setText(AppSettings->getValue("PieceString").toString());
 
+    ui.fontText->setText(AppSettings->getValue("FontBrowserText").toString());
+    ui.fontMove->setText(AppSettings->getValue("FontBrowserMove").toString());
+
+    AppSettings->endGroup();
+
+    AppSettings->beginGroup("Tools");
+    ui.extToolPath->setText(AppSettings->getValue("Path1").toString());
+    ui.extToolParameters->setText(AppSettings->getValue("CommandLine1").toString());
     AppSettings->endGroup();
 }
 
@@ -513,10 +534,10 @@ void PreferencesDialog::saveSettings()
     AppSettings->setValue("showFrame", QVariant(ui.boardFrameCheck->isChecked()));
     AppSettings->setValue("showCoordinates", QVariant(ui.boardShowCoordinates->isChecked()));
     AppSettings->setValue("showCurrentMove", QVariant(ui.hilightCurrentMove->isChecked()));
+    AppSettings->setValue("showMoveIndicator", QVariant(ui.cbShowIndicator->currentIndex()));
     AppSettings->setValue("guessMove", QVariant(ui.guessMoveCheck->isChecked()));
     AppSettings->setValue("nextGuess", QVariant(ui.guessNextMove->isChecked()));
     AppSettings->setValue("minWheelCount", ui.minWheelCount->value());
-    AppSettings->setValue("AutoPlayerInterval", ui.autoPlayInterval->value());
     AppSettings->setValue("pieceTheme", ui.pieceThemeCombo->currentText());
     AppSettings->setValue("pieceEffect", ui.pieceEffect->currentIndex());
     AppSettings->setValue("AutoSaveAndContinue", QVariant(ui.cbSaveAndContinue->isChecked()));
@@ -542,6 +563,7 @@ void PreferencesDialog::saveSettings()
     AppSettings->setValue("/General/DefaultDataPath", ui.defaultDataBasePath->text());
     AppSettings->setValue("/General/ListFontSize", ui.spinBoxListFontSize->value());
     AppSettings->setValue("/MainWindow/VerticalTabs", ui.verticalTabs->isChecked());
+    AppSettings->setValue("/MainWindow/ShowMenuIcons", ui.iconsVisible->isChecked());
 
     AppSettings->beginGroup("GameText");
 
@@ -552,23 +574,33 @@ void PreferencesDialog::saveSettings()
     AppSettings->setValue("DiagramSize", ui.diagramSize->value());
     AppSettings->setValue("PieceString", ui.pieceString->text());
 
+    AppSettings->setValue("FontBrowserText", ui.fontText->text());
+    AppSettings->setValue("FontBrowserMove", ui.fontMove->text());
+
     QStringList colorNamesNotation;
     colorNamesNotation << "MainLineMoveColor" << "VariationColor" << "CommentColor" << "NagColor";
     saveColorList(ui.notationColors, colorNamesNotation);
 
     AppSettings->endGroup();
 
+    AppSettings->beginGroup("Tools");
+    AppSettings->setValue("Path1", ui.extToolPath->text());
+    AppSettings->setValue("CommandLine1", ui.extToolParameters->text());
+    AppSettings->endGroup();
+
     QDir().mkpath(ui.defaultDataBasePath->text());
 }
 
 bool PreferencesDialog::selectInCombo(QComboBox* combo, const QString& text)
 {
     for(int i = 0; i < combo->count(); ++i)
+    {
         if(combo->itemText(i) == text)
         {
             combo->setCurrentIndex(i);
             return true;
         }
+    }
     combo->setCurrentIndex(combo->count() - 1);
     return false;
 }
@@ -586,3 +618,37 @@ void PreferencesDialog::saveColorList(ColorList* list, const QStringList& cfgnam
         AppSettings->setValue(cfgnames[i], list->color(i));
     }
 }
+
+void PreferencesDialog::slotBtMoveFontClicked()
+{
+    bool ok;
+    QFont currentFont = QFont(ui.fontMove->text());
+    QFont font = QFontDialog::getFont(&ok, currentFont, this);
+    if (ok)
+    {
+        QString fontFamily = font.family();
+        ui.fontMove->setText(fontFamily);
+    }
+}
+
+void PreferencesDialog::slotBtTextFontClicked()
+{
+    bool ok;
+    QFont currentFont = QFont(ui.fontText->text());
+    QFont font = QFontDialog::getFont(&ok, currentFont, this);
+    if (ok)
+    {
+        QString fontFamily = font.family();
+        ui.fontText->setText(fontFamily);
+    }
+}
+
+void PreferencesDialog::setupIconInMenus(QObject* pObject)
+{
+    QList<QAction*> actions = pObject->findChildren<QAction*>();
+    bool showIcons = AppSettings->getValue("/MainWindow/ShowMenuIcons").toBool();
+    foreach (QAction* action, actions)
+    {
+        action->setIconVisibleInMenu(showIcons);
+    }
+}
